/***************************************************************************
 *            browser.c
 *
 * Implements a service browser API.
 *
 *  Sun Mar  5 08:59:10 2006
 *  Copyright  2006  Darryl Luff
 *  Email darryl@snakegully.nu
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "browse_avahi.h"
#include "browser.h"
#include "dbapi.h"
#include "gtk_config.h"

static BrowseEntry *list = NULL;

static BrowseEntry *find_display(const char *name)
{
	BrowseEntry *b;

	if (!name)
		return NULL;

	b = list;
	while (b) {
		if (b->display) {
			if (!strcmp(name, b->display))
				return b;
		}
		b = b->next;
	}
	return NULL;
}

void browse_addItem(const char *url, const char *display)
{
	BrowseEntry *entry;
	BrowseEntry *parent;

	if ((!display) || (!url))
		return;

	if (find_display(display)) {
		fprintf(stderr, "WARNING: Browser not adding '%s': already present\n",
				display);
		return;
	}

	entry = calloc(1, sizeof(BrowseEntry));
	if (!entry) {
		fprintf(stderr, "WARNING: Browser could not add '%s': no memory\n",
				display);
		return;
	}

	entry->display = strdup(display);
	entry->url = strdup(url);

	if (list) {
		parent = list;
		while (parent->next)
			parent = parent->next;

		parent->next = entry;
	}
	else
		list = entry;
}

BrowseEntry *browse_getlist(void)
{
	return list;
}

void browse_cfg_init(void)
{
	/* XXX not implemented yet. Should retrieve the past list from the
	 * config file. */
}

void browse_cfg_refresh(void)
{
	/* XXX not implemented yet. Should retrieve the past list from the
	 * config file. */
	printf("XXX browse_cfg_refresh() not implemented\n");
}

void browse_refresh(void)
{
	int connectors;
	int i;
	DBConnector *conn;
	const char *typestr;

	browse_clear();
	browse_cfg_refresh();

	/* For each driver */
	connectors = dbconnectors_count();
	for (i = 0; i < connectors; i++) {
		conn = dbconnectors_get(i);
#ifdef USE_AVAHI
		if (gtkcfg_get_browse_avahi()) {
			typestr = dbconnector_get_service_type(conn);
			if (typestr) {
				printf("Browser checking %s\n", typestr);
				browse_avahi_for(typestr);
			}
		}
#endif
	}
}

static BrowseEntry *browse_entry_free(BrowseEntry * b)
{
	if (!b)
		return NULL;

	/* Free any trailing entries */
	browse_entry_free(b->next);

	if (b->display)
		free(b->display);

	if (b->url)
		free(b->url);

	free(b);

	return NULL;
}

void browse_clear(void)
{
	list = browse_entry_free(list);
}

void browse_init(void)
{
	browse_cfg_init();
#ifdef USE_AVAHI
	browse_avahi_init();
#endif
}
