
#include "../config.h"

#include <check.h>
#include <stdlib.h>
#include <string.h>

#include "forms.h"

#define SAMPLE_NAME "the name"
#define SAMPLE_VALUE "the value"

START_TEST(form_field_test_init)
{
	FormField *fld;
	/* 
FormField *fld_free(FormField *fld);
FormField *fld_new(const char *nm);
 */
	fld = fld_new(NULL);
	fail_if(fld == NULL, "Could not initialise field");
	fld = fld_free(fld);
	fail_if(fld != NULL, "Could not free field");
	fld = fld_free(NULL);
	fail_if(fld != NULL, "Could not free NULL field");
}
END_TEST

START_TEST(form_field_test_help)
{
	FormField *fld;
	/* const char *fld_get_help(const FormField * fld);	*/
	fld = fld_new(NULL);
	fail_if(fld == NULL, "Could not initialise field");
	// XXX now test the rest

	fld = fld_free(fld);
	fail_unless(fld == NULL, "fld_free() did not return NULL");
}
END_TEST

START_TEST(form_field_test_name)
{
	FormField *fld;
	const char *nm;

	/* const char *fld_get_name(const FormField * fld); */
	nm = fld_get_name(NULL);
	fail_if(nm != NULL, "Invalid name from NULL field");

	fld = fld_new(NULL);
	fail_if(fld == NULL, "Could not initialise field");
	nm = fld_get_name(fld);
	fail_if(nm != NULL, "Invalid name from empty init");
	fld = fld_free(fld);

	fld = fld_new(SAMPLE_NAME);
	nm = fld_get_name(fld);
	fail_if(nm == NULL, "Name not initialised");
	fail_unless(strcmp(nm, SAMPLE_NAME) == 0, "Name initialised incorrectly");
	fld = fld_free(fld);
}
END_TEST

START_TEST(form_field_test_relevance)
{
	FormField *fld;
	FieldRelevance rel;

	fld = fld_new(NULL);
	fail_if(fld == NULL, "Could not initialise field");

	/*
	const FieldRelevance fld_get_relevance(const FormField * fld);
	void fld_set_relevance(FormField * fld, const FieldRelevance r);
	*/
	rel = fld_get_relevance(NULL);
	fail_unless(rel == FR_OPTIONAL, "Bad relevence from NULL field");

	rel = fld_get_relevance(fld);
	fail_unless(rel == FR_OPTIONAL, "Bad relevence from initialised field");

	fld_set_relevance(NULL, 0);
	fld_set_relevance(NULL, FR_REQUIRED);
	fld_set_relevance(fld, FR_REQUIRED);
	rel = fld_get_relevance(fld);
	fail_unless(rel == FR_REQUIRED, "Relevence could not be set");
	
	fld_set_relevance(NULL, -99);
	rel = fld_get_relevance(fld);
	fail_unless(rel == FR_REQUIRED, "Relevence accepted negative value");

	fld_set_relevance(NULL, 1000);
	rel = fld_get_relevance(fld);
	fail_unless(rel == FR_REQUIRED, "Relevence accepted over value");

	fld = fld_free(fld);
}
END_TEST

START_TEST(form_field_test_tip)
{
	FormField *fld;
	const char *tip;
#define SAMPLE_TIP "thetip"
	/*
	const char *fld_get_tip(const FormField * fld);
	void fld_set_tip(FormField * fld, const char *txt);
	*/

	fld = fld_new(NULL);
	fail_if(fld == NULL, "Could not initialise field");

	tip = fld_get_tip(NULL);
	fail_unless(tip == NULL, "Bad tip from NULL field");

	tip = fld_get_tip(fld);
	fail_unless(tip == NULL, "Bad tip from initialised field");

	fld_set_tip(NULL, NULL);
	fld_set_tip(fld, NULL);
	tip = fld_get_tip(fld);
	fail_unless(tip == NULL, "Invalid tip after set");
	
	fld_set_tip(fld, SAMPLE_TIP);
	tip = fld_get_tip(fld);
	fail_unless(strcmp(tip, SAMPLE_TIP) == 0, "Tip not accepted");

	fld = fld_free(fld);
}
END_TEST

START_TEST(form_field_test_type)
{
	FormField *fld;
	FieldType typ;

	/*
	const FieldType fld_get_type(const FormField * fld);
	void fld_set_type(FormField * fld, const FieldType typ);
	*/

	fld = fld_new(NULL);
	fail_if(fld == NULL, "Could not initialise field");

	typ = fld_get_type(NULL);
	fail_unless(typ == FT_NONE, "Bad type from NULL field");

	typ = fld_get_type(fld);
	fail_unless(typ == FT_TEXT, "Bad type from initialised field");

	fld_set_type(NULL, 0);
	fld_set_type(fld, FT_FILENAME);
	typ = fld_get_type(fld);
	fail_unless(typ == FT_FILENAME, "Invalid type after set");

	fld_set_type(fld, -99);
	typ = fld_get_type(fld);
	fail_unless(typ == FT_FILENAME, "Type accepted negative value");

	fld_set_type(fld, 1000);
	typ = fld_get_type(fld);
	fail_unless(typ == FT_FILENAME, "Type accepted over-value");

	fld = fld_free(fld);
}
END_TEST

static void	add_field_tests(Suite *s) {
	TCase *tc;

	tc = tcase_create("Field");
	suite_add_tcase(s, tc);
	tcase_add_test(tc, form_field_test_init);
	tcase_add_test(tc, form_field_test_help);
	tcase_add_test(tc, form_field_test_name);
	tcase_add_test(tc, form_field_test_relevance);
	tcase_add_test(tc, form_field_test_tip);
	tcase_add_test(tc, form_field_test_type);
}

START_TEST(form_test_title)
{
	Form *frm;
	const char *ttl;
#define SAMPLE_TITLE "the title"
	/*
const char *frm_get_title(const Form * frm);
Form *frm_set_title(Form * f, const char *ttl);
	*/

	frm = frm_new(NULL);
	fail_if(frm == NULL, "Could not initialise form");

	ttl = frm_get_title(NULL);
	fail_unless(ttl == NULL, "Bad title from NULL field");

	frm_set_title(NULL, NULL);
	frm_set_title(frm, NULL);
	ttl = frm_get_title(frm);
	fail_unless(ttl == NULL, "Bad title from initialised form");

	frm_set_title(frm, SAMPLE_TITLE);
	ttl = frm_get_title(frm);
	fail_unless(strcmp(ttl, SAMPLE_TITLE) == 0, "New title not accepted");

	frm = frm_free(frm);
	fail_unless(frm == NULL, "frm_free() did not return NULL");
}
END_TEST

START_TEST(form_test_fields)
{
	Form *frm;
	const FormField *fld;
	ABuf *err;
	const char *nm;

/*
FormField *frm_add_field(Form * frm, const char *nm, const char *val);
int frm_get_field_count(const Form * frm);
const FormField *frm_get_field(const Form * frm, int n);
const char *frm_get_field_name(const Form * frm, int i);

ABuf *frm_check_fields(Form * frm);
void frm_clear_fields(Form * frm);
*/

	frm = frm_new(NULL);
	fail_if(frm == NULL, "Could not allocate form");

	fail_unless(frm_get_field_count(NULL) == 0, "Non-0 field count with NULL form");
	fail_unless(frm_get_field_count(frm) == 0, "Non-0 field count with initialised form");

	frm_add_field(NULL, NULL, NULL);
	frm_add_field(frm, NULL, NULL);
	fail_unless(frm_get_field_count(frm) == 1, "Added NULL field");
	frm_add_field(frm, SAMPLE_NAME, NULL);
	fail_unless(frm_get_field_count(frm) == 2, "Could not add field");

	fld = (FormField *)frm_get_field(NULL, 0);
	fail_unless(fld == NULL, "Bad return from frm_get_field with NULL form");

	fld = frm_get_field(frm, -1);
	fail_unless(fld == NULL, "Bad return from frm_get_field with negative index");

	fld = frm_get_field(frm, 99);
	fail_unless(fld == NULL, "Bad return from frm_get_field with over index");

	fld = frm_get_field(frm, 1);
	fail_if(fld == NULL, "Could not retrieve field");

	nm = fld_get_name(fld);
	fail_if(nm == NULL, "fld_get_name returned NULL for named field");
	fail_unless(strcmp(nm, SAMPLE_NAME) == 0, "Invalid name on fetched field");
	nm = frm_get_field_name(frm, 1);
	fail_if(nm == NULL, "frm_get_field_name returned NULL for named field");
	fail_unless(strcmp(nm, SAMPLE_NAME) == 0, "Invalid name from form");

	err = frm_check_fields(frm);
	fail_unless(err == NULL, "frm_check_fields() with optional field failed");
	err = buf_free(err);

	fld_set_relevance((FormField *)fld, FR_REQUIRED);
	fld_set_value((FormField *)fld, SAMPLE_VALUE);
	err = frm_check_fields(frm);
	fail_unless(err == NULL, "frm_check_fields() with present compulsory field failed");
	err = buf_free(err);

	fld_set_value((FormField *)fld, NULL);
	err = frm_check_fields(frm);
	fail_if(err == NULL, "frm_check_fields() with missing compulsory field failed");
	err = buf_free(err);

	frm = frm_free(frm);
	fail_unless(frm == NULL, "frm_free() did not return NULL");
}
END_TEST

START_TEST(form_test_value)
{
	Form *frm;
	const char *v;

	/*
const char *frm_get_value_by_name(const Form * frm, const char *nm);
void frm_set_value_by_name(Form * frm, const char *nm, const char *val);
	*/

	v = frm_get_value_by_name(NULL, NULL);
	fail_unless(v == NULL, "Invalid value from NULL form");

	frm = frm_new(NULL);
	fail_if(frm == NULL, "Could not initialise form");
	v = frm_get_value_by_name(frm, NULL);
	fail_if(v != NULL, "Invalid value from initialised form");

	frm_set_value_by_name(NULL, NULL, NULL);
	frm_set_value_by_name(frm, NULL, SAMPLE_VALUE);
	frm_set_value_by_name(frm, SAMPLE_NAME, NULL);
	v = frm_get_value_by_name(frm, SAMPLE_NAME);
	fail_unless(v == NULL, "Invalid value from uninitialised name");
	
	frm_set_value_by_name(frm, SAMPLE_NAME, SAMPLE_VALUE);
	v = frm_get_value_by_name(frm, SAMPLE_NAME);
	fail_if(v == NULL, "No value returned from initialised name");
	fail_unless(strcmp(v, SAMPLE_VALUE) == 0, "Invalid value from initialised name");

	frm = frm_free(frm);
}
END_TEST

static void	add_form_tests(Suite *s) {
	TCase *tc;

	tc = tcase_create("Form");
	suite_add_tcase(s, tc);
	tcase_add_test(tc, form_test_fields);
	tcase_add_test(tc, form_test_title);
	tcase_add_test(tc, form_test_value);
}

Suite * db_suite(void)
{
	Suite *s = suite_create("Forms");
	TCase *tc;

	tc = NULL;

	add_field_tests(s);
	add_form_tests(s);

	return s;
}

int main(void)
{
	int nf;

	Suite *s = db_suite();
	SRunner *sr = srunner_create(s);
	srunner_run_all(sr, CK_VERBOSE);
	nf = srunner_ntests_failed(sr);
	srunner_free(sr);
	return (nf == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
}
