/** \file d_sqlite_result.h	sqlite driver result structures.
 * \author Lionel ULMER
 * \author Darryl Luff
 * \addtogroup db */
/* GtkSQL -- an interactive graphical query tool for PostgreSQL, MySQL and
 * SQLite.
 * Copyright (C) 2003-2005 Darryl Luff
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __D_SQLITE_RESULT_H__
#define __D_SQLITE_RESULT_H__


typedef struct _resultvalue ResultValue;
/** One value from a result set.	*/
struct _resultvalue {
	/** Result text.	*/
	char *text;
	/** Next result value.	*/
	ResultValue *next;
};


typedef struct _resultrow ResultRow;
/** A single row of results.	*/
struct _resultrow {
	/** A value.	*/
	ResultValue *val;
	/** Next value.	*/
	ResultRow *next;
};

/** The result structure returned to the application.	*/
typedef struct _sqliteresult SQLiteResult;
/** The complete result structure.	*/
struct _sqliteresult {
	/** Column headers.	*/
	ResultRow *headers;
	/** Row data.	*/
	ResultRow *rows;
};

ResultValue *resultrow_append_field(ResultRow * row, char *txt);
ResultValue *sqliteresult_append_header(SQLiteResult * res, char *s);
ResultRow *sqliteresult_append_row(SQLiteResult * res);
void sqliteresult_clear_headers(SQLiteResult * res);
SQLiteResult *sqliteresult_free(SQLiteResult * res);
int sqliteresult_get_column_count(SQLiteResult * result);
char *sqliteresult_get_field(SQLiteResult * result, int rowno, int fieldno);
char *sqliteresult_get_header_text(SQLiteResult * result, int field);
int sqliteresult_get_row_count(SQLiteResult * result);
SQLiteResult *sqliteresult_new(void);

#endif
