/** \addtogroup db 
 *
 *@{
 */
/** \file db_keywords.h	Data structures and functions for managing 
 * the keyword list.
 * \author Lionel ULMER
 * \author Darryl Luff
 *
 */
/* GtkSQL -- an interactive graphical query tool for PostgreSQL and MySQL.
 * Copyright (C) 1998-2005  Lionel ULMER, Darryl Luff.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of tdo_he License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _DB_KEYWORDS_H
#define _DB_KEYWORDS_H

/** Types of keywords.	*/
typedef enum _keywordtype {
	KEYWORD_FREE,		/**< Unused keyword	*/
	KEYWORD_SQL,		/**< SQL keywords.	*/
	KEYWORD_TABLE,		/**< Table names.	*/
	KEYWORD_FIELD		/**< Field names.	*/
} KeywordType;

/** Auto-completion keyword list.	*/
typedef struct _keyword {
	char *key;				/**< Keyword text.	*/
	KeywordType type;		/**< Type of keyword.	*/
	struct _keyword *next;	/**< Pointer to next keyword in list.	*/
	struct _keyword *root;	/**< Pointer to the root of this chain.	*/
} Keyword;

/** \param key	The keyword list to add to.
 * \param type	\see KeywordType.
 * \param txt	The keyword to add.
 * \returns	The newly allocated Keyword object	*/
const Keyword *keyword_add(Keyword * key, const KeywordType type, const char *txt);
int keyword_count(const Keyword * key);

/** \param key	The keyword list to free.
 * \return	Always returns NULL.	*/
Keyword *keyword_free(Keyword * key);

/** Only checks for a match on the first strlen(check) characters of the keyword.
 * If the check string is NULL, matches EVERY key. Case insensitive.
 * \param	key	The key to check.
 * \param	check	The test string.
 * \return A pointer to the matching keyword record. NULL if no match. */
const Keyword *keyword_match(const Keyword * key, const char *check);

/** Checks for a match between the key and a string. Checks the whole
 * length of the key against the check string.
 * \param	key	The key to check.
 * \param	check	The test string. If NULL, matches every key.
 * \return A pointer to the matching keyword record. NULL if no match. */
const Keyword *keyword_match_exact(const Keyword * key, const char *check);
const Keyword *keyword_match_next(const Keyword * key, const char *check);
Keyword *keyword_new();
void keyword_remove_type(Keyword * key, const KeywordType type);

/*@}*/
#endif
