/** \addtogroup db
 *@{
 */
/** \file db_tabledef.h	Table definitions.
 * \author Lionel ULMER
 * \author Darryl Luff
 *
 * This file provides definitions of the Table definition.
 */
/* GtkSQL -- an interactive graphical query tool for PostgreSQL and MySQL.
 * Copyright (C) 1998-2004	Lionel ULMER, Darryl Luff.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of tdo_he License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _DB_TABLEDEF_H
#define _DB_TABLEDEF_H

/** Field structure.	*/
typedef struct _dbfielddef {
	char *fldName;				/**< Field name.	*/
	char *fldType;				/**< Field type string.	*/
	struct _dbfielddef *next;	/**< Link to next in list.	*/
} DBFieldDef;

/** Table structure.	*/
typedef struct _tabledef {
	char *name;				/**< Table name.	*/
	DBFieldDef *fieldDef;	/**< Field definitions.	*/
	struct _tabledef *next;	/**< Link to next in list.	*/
} DBTableDef;

const char *fielddef_getlength(const DBFieldDef * fld);
const char *fielddef_getname(const DBFieldDef * fld);
const char *fielddef_gettype(const DBFieldDef * fld);
DBFieldDef *fielddef_new(void);
void fielddef_setlength(DBFieldDef * fld, const char *length);
void fielddef_setname(DBFieldDef * fld, const char *name);
void fielddef_settype(DBFieldDef * fld, const char *type);
const DBTableDef *tabledef_add(DBTableDef * tbl, const DBTableDef * tbl2);
const DBFieldDef *tabledef_addfield(DBTableDef * tbl, const DBFieldDef * fld);
int tabledef_count(const DBTableDef * tbl);
int tabledef_fieldcount(const DBTableDef * tbl);
DBTableDef *tabledef_free(DBTableDef * tbl);
void tabledef_freefields(DBTableDef * tbl);
const DBTableDef *tabledef_gettable(const DBTableDef * tbl, int i);
DBFieldDef *tabledef_setfields(DBTableDef * tbl, int n);
void tabledef_setname(DBTableDef * tbl, const char *name);
DBTableDef *tabledef_new();
const DBFieldDef *tabledef_getfield(const DBTableDef * tbl, int i);
const char *tabledef_getname(const DBTableDef * tbl);

/*@}*/
#endif
