/** \file export.c	Functions to export results to files.
 * \author Lionel ULMER
 * \author Darryl Luff
 * \ingroup misc
 *
 * A number of output formats are supported.
 *
 * There are a number of dialogs supported in this module.
 */
/* GtkSQL -- an interactive graphical query tool for PostgreSQL
 * Copyright (C) 1998-2005  Lionel ULMER, Darryl Luff.
 *
 * This program is free software; you can redistribute it andraw/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/** \todo Consolidate options and provide a way to query values and text representations.
 * Options currently used are:
 */

#include "../config.h"
#include <stdlib.h>
#include <string.h>
#include "export.h"
#include "guiapi.h"


ExportOptions exportOptions = {
	NULL,
	100	
};

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

#define DEFAULT_SEPARATOR	","
#define DEFAULT_RECSPERPAGE	100


/** Carry out the HTML export.
 * \param dedata	The data required to do the export.	*/
void do_export_html(DoExportData *dedata)
{
/** Filename template for the HTML files.	*/
#define HTMLFILE_S	"%s%03d.html"
	FILE *f = NULL;
	char *bsname = NULL;
	char *lbase = NULL;
	ABuf *current_file = NULL;
	int currentPage, recPerPage;
	int nb_tuples;
	int nb_cols;
	int i, j;
	int col;

	if (dbconnection_get_query_status(dedata->conn, dedata->result) != DB_QUERY_RECORDS_OK)
		return;

	nb_tuples = dbconnection_get_record_count(dedata->conn, dedata->result);
	nb_cols = dbconnection_get_field_count(dedata->conn, dedata->result);
	recPerPage = exportOptions.htmlRecordsPerPage;
	if (recPerPage < 1) {
		fprintf(stderr, "Invalid records per page (%d)\n", recPerPage);
		return;
	}

	if (recPerPage < nb_tuples) {
		bsname = strdup(dedata->filename);
		for (i = strlen(bsname); (i > 0) && (bsname[i] != '.'); i--)
			;

		bsname[i] = '\0';
		for (j = i; (j > 0) && (bsname[j] != '/'); j--)
			;

		lbase = strdup(bsname + j + 1);
	}

	/* Start exporting */
	ui_status_push("Exporting HTML...");
	ui_progress_reset(dedata->query);
	currentPage = 0;

	for (i = 0; i < nb_tuples; i++) {
		if ((i % recPerPage) == 0) {
			if (recPerPage < nb_tuples) {
				if (currentPage != 0) {
					fprintf(f, "</TABLE>\n");
					fclose(f);
				}
				current_file = buf_check(current_file,
					strlen(HTMLFILE_S) + strlen(bsname) + 50);
				snprintf((char *)current_file->b_dat, current_file->b_len, HTMLFILE_S,
					bsname, currentPage++);
				f = fopen((char *)current_file->b_dat, "w");

				if (currentPage != 1)
					fprintf(f, "<A HREF=\"%s%03d.html\">Previous</A> ", lbase, currentPage - 2);

				if (nb_tuples > (currentPage + 1) * recPerPage)
					fprintf(f, " <A HREF=\"%s%03d.html\">Next</A>", lbase, currentPage);

				fprintf(f, "%s", "<TABLE border>\n");
				if (dedata->caption) {
					if (currentPage == 0)
						fprintf(f, "<CAPTION>%s</CAPTION>\n", dedata->caption);
					else
						fprintf(f, "<CAPTION>%s (continued...)</CAPTION>\n", dedata->caption);
				}

				fprintf(f, "%s", " <TR>\n");
				for (col = 0; col < nb_cols; col++) {
					fprintf(f, "  <TH>%s</TH>\n",
						dbconnection_get_field_name(dedata->conn, dedata->result, col));
				}
				fprintf(f, "%s", " </TR>\n");
			}
			else if (i == 0) {
				f = fopen(dedata->filename, "w");
				fprintf(f, "%s", "<TABLE border>\n");
				if (dedata->caption)
					fprintf(f, "<CAPTION>%s</CAPTION>\n", dedata->caption);

				fprintf(f, "%s", " <TR>\n");
				for (col = 0; col < nb_cols; col++) {
					fprintf(f, "  <TH>%s</TH>\n",
							dbconnection_get_field_name(dedata->conn, dedata->result, col));
				}
				fprintf(f, "%s", " </TR>\n");
			}
		}

		/* Exports the line */
		fprintf(f, "%s", " <TR>\n");
		for (j = 0; j < nb_cols; j++) {
			fprintf(f, "  <TD> %s </TD>\n",
				dbconnection_get_field_value(dedata->conn, dedata->result, i, j));
		}
		ui_progress_update(dedata->query, i / (float) nb_tuples);
		fprintf(f, "%s", " </TR>\n");
	}

	/* Closes the file */
	fprintf(f, "%s", "</TABLE>\n");
	fclose(f);

	ui_progress_reset(dedata->query);
	ui_status_pop(dedata->query);

	/* Memory cleaning */
	free(bsname);
	buf_free(current_file);
}

void do_export_text(DoExportData *dedata) {
	FILE *f;
	char *sep;
	const char *field;

	if ((dedata == NULL) || (dedata->filename == NULL)) {
		fprintf(stderr, "Null filename entered\n");
		return;
	}

	sep = exportoptions_get_separator();

	f = fopen(dedata->filename, "wb");
	if (f == NULL) {
		perror("opening export file");
		return;
	}

	/* Start exporting ... */
	if ((dedata->result) &&
		(dbconnection_get_query_status(dedata->conn, dedata->result) == DB_QUERY_RECORDS_OK)) {
		int nb_tuples = dbconnection_get_record_count(dedata->conn, dedata->result);
		int nb_cols = dbconnection_get_field_count(dedata->conn, dedata->result);
		int *col_length = (int *) malloc(sizeof(int) * nb_cols);
		int i, j;

		if (nb_tuples > 0) {
			ui_status_push("Exporting Text...");
			ui_progress_reset(dedata->query);

			/* Print the data */
			for (i = 0; i < nb_tuples; i++) {
				for (j = 0; j < nb_cols; j++) {
					if (j > 0)
						fprintf(f, "%s", sep);

					field = dbconnection_get_field_value(dedata->conn, dedata->result, i, j);
					if (!field)
						field = "";

					fprintf(f, "%s", field);
				}
				ui_progress_update(dedata->query, i / (float) nb_tuples);
				fprintf(f, "\n");
			}
			free(col_length);
		}
	}

	/* Closes the file */
	fclose(f);

	/* Close the dialog */
	ui_progress_reset(dedata->query);
	ui_status_pop(dedata->query);
}

void exportoptions_set_separator(const char *sep) {
	if (exportOptions.separator) {
		free(exportOptions.separator);
		exportOptions.separator = NULL;
	}

	if (sep)
		exportOptions.separator = strdup(sep);
}

char *exportoptions_get_separator(void) {
	if (! exportOptions.separator)
		exportoptions_set_separator(DEFAULT_SEPARATOR);

	return exportOptions.separator;
}
