/** \file export.h	Include file for export functions.
 * \author Lionel ULMER
 * \author Darryl Luff
 * \addtogroup misc	*/
/* GtkSQL -- an interactive graphical query tool for PostgreSQL
 * Copyright (C) 1998-2004  Lionel ULMER, Darryl Luff.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __EXPORT_H__
#define __EXPORT_H__

#include "dbapi.h"

/** Data required for the export functions.	*/
typedef struct
{
	char *caption;			/**< Caption for the export types that support it	*/
	DBConnection *conn;		/**< The connection the data relates to.	*/
	char *filename;			/**< Filename with path.	*/
	char *raw_filename;		/**< Raw filename without path.	*/
	void *result;			/**< The query result to dump.	*/
	void *query;			/**< The specific query to export. The contents of this pointer is ui driver specific.	*/
} DoExportData;

/** Options controlling export output.	*/
typedef struct
{
	char *separator;		/**< String to place between records.	*/
	int htmlRecordsPerPage;	/**< Records per page in the HTML output.	*/
} ExportOptions;

extern ExportOptions exportOptions;

void do_export_html (DoExportData * edata);
void do_export_text (DoExportData * edata);

char *exportoptions_get_separator (void);
void exportoptions_save_dlg (void);
void exportoptions_set_separator (const char *sep);

#endif
