/** \addtogroup uigtk
 *@{
 */
/** \file gtk_callbacks.c	GTK callbacks.
 * \author Darryl Luff	*/
/* GtkSQL -- an interactive graphical query tool for PostgreSQL
 * Copyright (C) 2003-2006 Darryl Luff
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include "appconfig.h"
#include "db_connect_dlg.h"
#include "export.h"
#include "gtk_callbacks.h"
#include "gtk_config.h"
#include "gtk_export.h"
#include "gtk_queries.h"
#include "gtk_interface.h"
#include "gtk_support.h"
#include "gtk_treeview.h"
#include "gtk_workspace.h"
#include "gtksql_query_scintilla.h"
#include "gtkui-object.h"
#include "gui_gtk.h"
#include "scriptapi.h"
#include "utils.h"

void on_cut_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	GtkWidget *widget;
	GtkTextBuffer *buffer;
	GdkAtom clipatom;
	GtkClipboard *clipboard;
	GtkWidget *mainWindow;

	mainWindow = gtkui_get_mainWindow(gtkapp);

	widget = gtk_window_get_focus(GTK_WINDOW(mainWindow));
	if (GTK_WIDGET_HAS_FOCUS(widget)) {
		/* This could be a GtkTextView or a GtkEditable, GtkTreeView */
		if (IS_SCINTILLA(widget))
			gtksql_query_scintilla_do_cut(widget);
		else if (GTK_TREE_VIEW(widget)) {
			/* Do nothing. Shouldnt be called.	*/
		}
		else if (GTK_IS_TEXT_VIEW(widget)) {
			buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
			if (!buffer)
				return;

			clipatom = gdk_atom_intern("CLIPBOARD", TRUE);
			if (!clipatom)
				return;

			clipboard = gtk_clipboard_get(clipatom);
			if (!clipboard)
				return;

			gtk_text_buffer_cut_clipboard(buffer, clipboard, TRUE);
		}
		else if (GTK_IS_EDITABLE(widget)) {
			widget = GTK_WIDGET(GTK_EDITABLE(widget));
			if (widget)
				gtk_editable_cut_clipboard(GTK_EDITABLE(widget));
		}
		else
			g_warning("Unknown widget type for cut\n");
	}
}

void on_copy_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	GtkWidget *widget;
	GtkTextBuffer *buffer;
	GdkAtom clipatom;
	GtkClipboard *clipboard;
	GtkWidget *mainWindow;

	mainWindow = gtkui_get_mainWindow(gtkapp);

	widget = gtk_window_get_focus(GTK_WINDOW(mainWindow));
	if (GTK_WIDGET_HAS_FOCUS(widget)) {
		/* This could be a Scintilla, GtkTreeView, GtkTextView or a GtkEditable */
		if (IS_SCINTILLA(widget))
			gtksql_query_scintilla_do_copy(widget);
		else if (GTK_TREE_VIEW(widget))
			treeview_do_copy(widget);
		else if (GTK_IS_TEXT_VIEW(widget)) {
			buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
			if (!buffer)
				return;

			clipatom = gdk_atom_intern("CLIPBOARD", TRUE);
			if (!clipatom)
				return;

			clipboard = gtk_clipboard_get(clipatom);
			if (!clipboard)
				return;

			gtk_text_buffer_copy_clipboard(buffer, clipboard);
		}
		else if (GTK_IS_EDITABLE(widget)) {
			widget = GTK_WIDGET(GTK_EDITABLE(widget));
			if (widget)
				gtk_editable_copy_clipboard(GTK_EDITABLE(widget));
		}
	}
}

void on_queryNew_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	ui_add_query();
}

void on_open_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	query_add();
	query_import();
}

void on_paste_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	GtkWidget *widget;
	GtkTextBuffer *buffer;
	GdkAtom clipatom;
	GtkClipboard *clipboard;
	GtkWidget *mainWindow;

	mainWindow = gtkui_get_mainWindow(gtkapp);

	widget = gtk_window_get_focus(GTK_WINDOW(mainWindow));
	if (GTK_WIDGET_HAS_FOCUS(widget)) {
		/* This could be a Scintilla, a GtkTextView or a GtkEditable */
		if (IS_SCINTILLA(widget))
			gtksql_query_scintilla_do_paste(widget);
		else if (GTK_IS_TEXT_VIEW(widget)) {
			buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
			if (!buffer)
				return;

			clipatom = gdk_atom_intern("CLIPBOARD", TRUE);
			if (!clipatom)
				return;

			clipboard = gtk_clipboard_get(clipatom);
			if (!clipboard)
				return;

			gtk_text_buffer_paste_clipboard(buffer, clipboard, NULL, TRUE);
		}
		else if (GTK_IS_EDITABLE(widget)) {
			widget = GTK_WIDGET(GTK_EDITABLE(widget));
			if (widget)
				gtk_editable_paste_clipboard(GTK_EDITABLE(widget));
		}
	}
}

void on_quit_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	cfg_save_file();
	ui_disconnect();
	//cfg_save_file();
	gtk_main_quit();
}

void on_querySave_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	query_save(NULL);
}

void on_mnuViewRefresh_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	on_refresh_activate(NULL, NULL);
}

void on_about_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	gtkui_show_about_dialog(gtkapp);
}

void on_querySaveAs_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	query_save_as(NULL);
}

void on_querySend_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	ui_send_query();
}

void on_queryClear_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	ui_set_query_text("");
}

void do_export_dialog(void) {
	GtkWidget *dialog;
	GtkWidget *htmlRadio;
	GtkWidget *textRadio;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtksqlQueryScintilla *qry;
	gboolean textExport;
	char *filename;

	dialog = gtk_file_chooser_dialog_new ("Save File", 
		NULL, GTK_FILE_CHOOSER_ACTION_SAVE,
		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);

	/*-- Create the first radio button with a label --*/
	gtkui_exportoptions_load_config(gtkapp);

	textRadio = gtk_radio_button_new_with_label (NULL, "Export as Text");
	htmlRadio = gtk_radio_button_new_with_label (gtk_radio_button_get_group(GTK_RADIO_BUTTON (textRadio)), "Export as HTML");
	vbox = gtk_vbox_new(FALSE, 1);
	gtk_container_add(GTK_CONTAINER(vbox), textRadio);
	gtk_container_add(GTK_CONTAINER(vbox), htmlRadio);
	gtk_widget_show(textRadio);
	gtk_widget_show(htmlRadio);
	gtk_widget_show(vbox);
	hbox = gtk_hbox_new(FALSE, 1);
	gtk_container_add(GTK_CONTAINER(hbox), vbox);
	gtk_widget_show(hbox);
	gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(dialog), hbox);

	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), "/tmp");
	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), "Untitled document");

	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		exportoptions_save_dlg();

		/* Setup the data block.  */
		qry = query_get_current();
		textExport = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(textRadio));
		gtkui_do_export(gtkapp, qry, filename, textExport);
		update_ui();
		g_free (filename);
	}
	gtk_widget_destroy (dialog);
}

void on_resultExport_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	do_export_dialog();
}

void on_mnuViewToolbar_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	gtkui_set_toolbar_visible(gtkapp, GTK_CHECK_MENU_ITEM(menuitem)->active);
}

void on_queryClose_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	query_close();
	update_ui();
}

void on_connect_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	db_connect_dialog_show();
}

void on_disconnect_activate(GtkMenuItem * menuitem, gpointer user_data)
{
/** Template for disconnection message.	*/
#define SURE_S	" Are you sure you want \n to disconnect from database %s ? "
	GtkWidget *dialog;
	DBConnection *conn = ui_get_connection();
	ABuf *buf =
		buf_new(strlen(SURE_S) + strlen(dbconnection_get_db_name(conn)));

	if (!buf) {
		g_warning("disconnect() could not allocate memory for a buffer\n");
		return;
	}

	if (conn == NULL)
		return;

	snprintf((char *)buf->b_dat, buf->b_len, SURE_S, dbconnection_get_db_name(conn));

	dialog =
		gtkui_create_question_dialog("Disconnect", (char *)buf->b_dat, "Ok", gtkui_disconnect_callback, "Cancel",
									 NULL, gtkapp);
	gtk_widget_show(dialog);
	buf_free(buf);
}

void on_refresh_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	treeview_new_connection();
}

void on_tbConnectBtn_clicked(GtkButton * button, gpointer user_data)
{
	on_connect_activate(NULL, NULL);
}

void on_tbDisconnectBtn_clicked(GtkButton * button, gpointer user_data)
{
	on_disconnect_activate(NULL, NULL);
}

void on_tbSendBtn_clicked(GtkButton * button, gpointer user_data)
{
	on_querySend_activate(NULL, NULL);
}

void on_tbNewBtn_clicked(GtkButton * button, gpointer user_data)
{
	on_queryNew_activate(NULL, NULL);
}

void on_tbLoadBtn_clicked(GtkButton * button, gpointer user_data)
{
	on_open_activate(NULL, NULL);
}

void on_tbSaveAsBtn_clicked(GtkButton * button, gpointer user_data)
{
	on_querySaveAs_activate(NULL, NULL);
}

void on_tbCloseBtn_clicked(GtkButton * button, gpointer user_data)
{
	on_queryClose_activate(NULL, NULL);
}

void on_tbExportBtn_clicked(GtkButton * button, gpointer user_data)
{
	do_export_dialog();
}

void on_tbRefreshBtn_clicked(GtkButton * button, gpointer user_data)
{
	on_refresh_activate(NULL, NULL);
}

void on_querySaveOKBtn_clicked(GtkButton * button, gpointer user_data)
{
	GtksqlQueryScintilla *query;

	query = query_get_current();
	gtkui_do_save_to_file(gtkapp, query);
}

void on_querySaveCancelBtn_clicked(GtkButton * button, gpointer user_data)
{
	//gtkui_hide_script_save_dialog(gtkapp);
}

void on_queryImportOKBtn_clicked(GtkButton * button, gpointer user_data)
{
	gtkui_do_load_from_file(gtkapp);
}

void on_queryImportCanelBtn_clicked(GtkButton * button, gpointer user_data)
{
	gtkui_hide_query_import_dialog(gtkapp);
}

void on_aboutCloseBtn_clicked(GtkButton * button, gpointer user_data)
{
	gtkui_hide_aboutDlg(gtkapp);
}

void on_mnuPreferences_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	gtkcfg_show_dialog();
}

void on_main_destroy_event(GtkObject * object, gpointer user_data)
{
	DBConnection *conn = ui_get_connection();

	if (conn != NULL)
		gtkui_disconnect_callback(user_data);

	cfg_save_file();

	gtk_main_quit();
}

void on_resultPopupPrint_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	unimp("on_resultPopupPrint_activate");
}

void on_mnuViewTreeview_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	gtkgui_show_table_treeview(GTK_CHECK_MENU_ITEM(menuitem)->active);
}

void on_queryOpen_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	query_add();
	query_import();
}

void on_resultPrint_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	unimp("on_resultPrint_activate()");
}

gboolean
on_main_delete_event(GtkWidget * widget, GdkEvent * event, gpointer user_data)
{
	return FALSE;
}

void on_treeSelectQuery_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	char buff[2048] = "";
	GtksqlQueryScintilla *query = NULL;
	char *tables = treeview_get_selected_tables();
	gchar *fields = treeview_get_selected_fields();

	if (!fields) {
		g_warning("Invalid fields\n");
		return;
	}

	if (!tables) {
		g_warning("Invalid tablename\n");
		return;
	}

	snprintf(buff, sizeof(buff), "select %s\nfrom %s\n", fields, tables);

	query = query_get_current();
	if (query == NULL)
		return;

	gtksql_query_scintilla_set_text(query, buff);

	g_free(fields);
	g_free(tables);

}

void on_treeInsertQuery_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	ABuf *buff = NULL;
	GtksqlQueryScintilla *query = NULL;

	gchar *tables = treeview_get_selected_tables();
	gchar *fields = treeview_get_selected_fields();

	if (!fields) {
		g_warning("Invalid fields\n");
		return;
	}

	if (!tables) {
		g_warning("Invalid tablename\n");
		return;
	}

	buff = buf_strcpy(buff, "insert into ");
	buff = buf_strcat(buff, tables);

	if (strlen(fields) > 0) {
		buff = buf_strcat(buff, " (");
		buff = buf_strcat(buff, fields);
		buff = buf_strcat(buff, ") ");
	}

	buff = buf_strcat(buff, "\nvalues (...)\n");

	query = query_get_current();
	if (query == NULL)
		return;

	gtksql_query_scintilla_set_text(query, (char *)buff->b_dat);

	g_free(fields);
	g_free(tables);

	buff = buf_free(buff);
}

void on_treeUpdateQuery_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	ABuf *buff = NULL;
	GtksqlQueryScintilla *query = NULL;
	gchar *tables = treeview_get_selected_tables();
	gchar *fields = treeview_get_selected_fields();

	if (!fields) {
		g_warning("Invalid fields\n");
		return;
	}

	if (!tables) {
		g_warning("Invalid tablename\n");
		return;
	}

	buff = buf_strcpy(buff, "update ");
	buff = buf_strcat(buff, tables);
	buff = buf_strcat(buff, "\nset ...");

	query = query_get_current();
	if (query == NULL)
		return;

	gtksql_query_scintilla_set_text(query, (char *)buff->b_dat);

	g_free(fields);
	g_free(tables);

	buff = buf_free(buff);
}

void on_treeDeleteQuery_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	ABuf *buff = NULL;
	GtksqlQueryScintilla *query = NULL;
	char *tables = treeview_get_selected_tables();
	gchar *fields = treeview_get_selected_fields();

	if (!fields) {
		g_warning("Invalid fields\n");
		return;
	}

	if (!tables) {
		g_warning("Invalid tablename\n");
		return;
	}

	buff = buf_strcpy(buff, "delete from ");
	buff = buf_strcat(buff, tables);
	buff = buf_strcat(buff, "\nwhere ...");

	query = query_get_current();
	if (query == NULL)
		return;

	gtksql_query_scintilla_set_text(query, (char *)buff->b_dat);
	g_free(fields);
	g_free(tables);

	buff = buf_free(buff);
}

void on_mnuScriptSaveAs_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	unimp("on_mnuScriptSaveAs_activate()");
}


void on_undo_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	GtksqlQueryScintilla *query = query_get_current();
	gtksql_query_scintilla_undo(query);
}

void on_redo_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	GtksqlQueryScintilla *query = query_get_current();
	gtksql_query_scintilla_redo(query);
}

/*@}{ */
