/** \file gtk_export.c	GTK export GUI.
 * \author Lionel ULMER
 * \author Darryl Luff
 * \addtogroup uigtk
 *
 * A number of output formats are supported.
 *
 * There are a number of dialogs supported in this module.
 */
/* GtkSQL -- an interactive graphical query tool for PostgreSQL
 * Copyright (C) 1998-2006 Lionel ULMER, Darryl Luff.
 *
 * This program is free software; you can redistribute it andraw/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/** \todo Consolidate options and provide a way to query values and text representations.
 * Options currently used are:
 *
 * TEXT:
 * Pretty print Y/N		This is OK.
 * Seperator char ','	This is OK.
 *
 * HTML:
 * Records per page		This is OK
 * Include links		[Why would you NOT want this in HTML?]
 * Include column titles	[Why would you NOT want this in HTML?]
 *
 * PROPOSED CONSOLIDATED OPTIONS:
 * Pretty Print:	In text, does whatever it does now. In HTML, 
 * Text seperator char.
 * HTML Records per page.
 * 
 */

#include "../config.h"
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include "export.h"
#include "gtk_interface.h"
#include "gtk_queries.h"
#include "gtkui-object.h"
#include "gui_gtk.h"
#include "utils.h"

/** Key for the GTK data field linked to the file extension menu.	*/
#define TYPE_KEY	"Type key"
#define QUERY_KEY	"Query"

/** Data for common export dialog.	*/
typedef struct _common_export_data {
	GtkWidget *dialog;	/**< Dialog widget.	*/
	char *filename;		/**< Export filename. This is the full filename including the full path information.	*/
	char *raw_filename;	/**< Raw filename. This is the base filename without the path.	*/
} CommonExportData;


/** Adds an entry line.	*/
/*static void
add_entry_line(char *labeltext, GtkWidget * widget,
			   GtkWidget * table, int height)
{
	GtkWidget *label;

	label = gtk_label_new(labeltext);
	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1,
							  height, height + 1);
	gtk_widget_show(label);

	gtk_table_attach_defaults(GTK_TABLE(table), widget, 1, 2,
							  height, height + 1);
}*/

/** Load the export options dialog from the config data.	*/
void exportoptions_save_dlg(void)
{
	GtkWidget *exportDlgTextSeparator = gtkui_get_exportDlgTextSeparator(gtkapp);
	GtkWidget *exportDlgHtmlRecordsPerPage = gtkui_get_exportDlgHtmlRecordsPerPage(gtkapp);

	if (exportDlgTextSeparator)
		exportoptions_set_separator(gtk_entry_get_text
									(GTK_ENTRY(exportDlgTextSeparator)));

	if (exportDlgHtmlRecordsPerPage)
		exportOptions.htmlRecordsPerPage =
			gtk_spin_button_get_value(GTK_SPIN_BUTTON
									  (exportDlgHtmlRecordsPerPage));
}
