/** \file gtk_queries.h	Interface to query display module.
 * \author Lionel ULMER
 * \author Darryl Luff
 * \addtogroup uigtk	*/
/* GtkSQL -- an interactive graphical query tool for PostgreSQL
 * Copyright (C) 1998-2006  Lionel ULMER, Darryl Luff.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * ==========================================================================
 *
 * Functions for managing the query and result windows.
 */

#ifndef __QUERIES_H__
#define __QUERIES_H__

#include "gtksql_query_scintilla.h"


/** Maximum result column width  */
#define MAXCOLWID 25

#define QUERY_QRY_KEY	"Query"

typedef enum {
	QUERY_NEW = 0,
	QUERY_FAKE,
	QUERY_ERROR
} QueryDisplay;

extern int queryStatus;

gint query_close(void);
void *query_get_current_result(void);
void queries_init(void);
void query_import(void);
int query_count(void);
void query_add(void);

GtksqlQueryScintilla *query_get_current(void);
GtksqlQueryScintilla *query_get_at(int n);
const gchar *query_get_name(GtksqlQueryScintilla * qry);
const char *query_get_text(GtksqlQueryScintilla * query);
void query_save(GtksqlQueryScintilla * qry);
void query_save_as(GtksqlQueryScintilla * qry);
void query_save_to_file(GtksqlQueryScintilla * query, const char *fn);
void query_set_name(GtksqlQueryScintilla * qry, const gchar * name);

#endif
