/** \file gtk_widgets.c	Widget references.
 * \author Darryl Luff
 * \addtogroup uigtk	*/
/* GtkSQL -- an interactive graphical query tool for PostgreSQL
 * Copyright (C) 2003-2005 Darryl Luff
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include "gtk_interface.h"
#include "gtk_support.h"

GtkWidget *aboutDlg = NULL;
GtkWidget *configDlg = NULL;
GtkWidget *connectorNotebook = NULL;
GtkWidget *mainWindow = NULL;
GtkWidget *exportDlg = NULL;
GtkWidget *exportDlgHtmlRadio = NULL;
GtkWidget *exportDlgHtmlRecordsPerPage = NULL;
GtkWidget *exportDlgTextRadio = NULL;
GtkWidget *exportDlgTextSeparator = NULL;
GtkWidget *menuEdit = NULL;
GtkWidget *menuFile = NULL;
GtkWidget *menuHelp = NULL;
GtkWidget *menuView = NULL;
GtkWidget *mnuAbout = NULL;
GtkWidget *mnuClear = NULL;
GtkWidget *mnuClose = NULL;
GtkWidget *mnuCopy = NULL;
GtkWidget *mnuConnect = NULL;
GtkWidget *mnuCut = NULL;
GtkWidget *mnuDisconnect = NULL;
GtkWidget *mnuExport = NULL;
GtkWidget *mnuNew = NULL;
GtkWidget *mnuOpen = NULL;
GtkWidget *mnuPaste = NULL;
GtkWidget *mnuPrintResults = NULL;
GtkWidget *mnuSaveAs = NULL;
GtkWidget *mnuScriptRun = NULL;
GtkWidget *mnuScriptSave = NULL;
GtkWidget *mnuScriptSaveAs = NULL;
GtkWidget *mnuSend = NULL;

GtkWidget *mnuViewRefresh = NULL;
GtkWidget *mnuViewToolbar = NULL;
GtkWidget *mnuViewTreeview = NULL;

GtkWidget *openFileDialog = NULL;

GtkWidget *queryNotebook = NULL;

GtkWidget *queryPopupClear = NULL;
GtkWidget *queryPopupClose = NULL;
GtkWidget *queryPopupSave = NULL;
GtkWidget *queryPopupSaveAs = NULL;
GtkWidget *queryPopupSend = NULL;

GtkWidget *querySaveDialog = NULL;

GtkWidget *resultPopup = NULL;
GtkWidget *resultPopupExport = NULL;
GtkWidget *resultPopupPrint = NULL;

GtkWidget *queryImportDlg = NULL;
GtkWidget *querySaveDlg = NULL;
GtkWidget *querySavePromptDlg = NULL;
GtkWidget *scriptOpenDlg = NULL;
GtkWidget *scriptSaveDlg = NULL;
GtkWidget *treeScroller = NULL;
GtkWidget *treeView = NULL;

GtkWidget *tablePopup = NULL;

GtkWidget *toolbarHandlebox = NULL;
GtkWidget *tbConnectBtn = NULL;
GtkWidget *tbCloseBtn = NULL;
GtkWidget *tbDisconnectBtn = NULL;
GtkWidget *tbExportBtn = NULL;
GtkWidget *tbLoadBtn = NULL;
GtkWidget *tbNewBtn = NULL;
GtkWidget *tbRefreshBtn = NULL;
GtkWidget *tbSaveAsBtn = NULL;
GtkWidget *tbSendBtn = NULL;
GtkWidget *toolbarHolder = NULL;
GtkWidget *treePopup = NULL;
GtkWidget *treePopupDelete = NULL;
GtkWidget *treePopupInsert = NULL;
GtkWidget *treePopupRefresh = NULL;
GtkWidget *treePopupSelect = NULL;
GtkWidget *treePopupUpdate = NULL;

void loadWidgets(void)
{
	toolbarHandlebox = lookup_widget(mainWindow, "toolbarHandlebox");
	menuEdit = lookup_widget(mainWindow, "menuEdit");
	menuFile = lookup_widget(mainWindow, "menuFile");
	menuHelp = lookup_widget(mainWindow, "menuHelp");
	menuView = lookup_widget(mainWindow, "menuView");
	mnuAbout = lookup_widget(mainWindow, "mnuAbout");
	mnuClear = lookup_widget(mainWindow, "mnuClear");
	mnuClose = lookup_widget(mainWindow, "mnuClose");
	mnuConnect = lookup_widget(mainWindow, "mnuConnect");
	mnuCopy = lookup_widget(mainWindow, "mnuCopy");
	mnuCut = lookup_widget(mainWindow, "mnuCut");
	mnuDisconnect = lookup_widget(mainWindow, "mnuDisconnect");
	mnuExport = lookup_widget(mainWindow, "mnuExport");
	mnuNew = lookup_widget(mainWindow, "mnuNew");
	mnuOpen = lookup_widget(mainWindow, "mnuOpen");
	mnuPaste = lookup_widget(mainWindow, "mnuPaste");
	mnuPrintResults = lookup_widget(mainWindow, "mnuPrintResults");
	mnuSaveAs = lookup_widget(mainWindow, "mnuSaveAs");
	mnuSend = lookup_widget(mainWindow, "mnuSend");
	mnuScriptRun = lookup_widget(mainWindow, "mnuScriptRun");
	mnuScriptSave = lookup_widget(mainWindow, "mnuScriptSave");
	mnuScriptSaveAs = lookup_widget(mainWindow, "mnuScriptSaveAs");

	mnuViewRefresh = lookup_widget(mainWindow, "mnuViewRefresh");
	mnuViewRefresh = lookup_widget(mainWindow, "mnuViewRefresh");
	mnuViewToolbar = lookup_widget(mainWindow, "mnuViewToolbar");
	mnuViewTreeview = lookup_widget(mainWindow, "mnuViewTreeview");

	queryNotebook = lookup_widget(mainWindow, "queryNotebook");

	querySaveDlg = create_querySaveDlg();
	queryImportDlg = create_queryImportDlg();

	resultPopup = create_resultPopup();
	if (resultPopup != NULL) {
		/* Lookup widgets.  */
		resultPopupExport = lookup_widget(resultPopup, "resultPopupExport");
		resultPopupPrint = lookup_widget(resultPopup, "resultPopupPrint");
	}

	scriptOpenDlg = create_scriptOpenDlg();
	scriptSaveDlg = create_scriptSaveDlg();

	tbConnectBtn = lookup_widget(mainWindow, "tbConnectBtn");
	tbCloseBtn = lookup_widget(mainWindow, "tbCloseBtn");
	tbDisconnectBtn = lookup_widget(mainWindow, "tbDisconnectBtn");
	tbExportBtn = lookup_widget(mainWindow, "tbExportBtn");
	tbLoadBtn = lookup_widget(mainWindow, "tbLoadBtn");
	tbNewBtn = lookup_widget(mainWindow, "tbNewBtn");
	tbRefreshBtn = lookup_widget(mainWindow, "tbRefreshBtn");
	tbSaveAsBtn = lookup_widget(mainWindow, "tbSaveAsBtn");
	tbSendBtn = lookup_widget(mainWindow, "tbSendBtn");

	toolbarHolder = lookup_widget(mainWindow, "toolbarHolder");

	treeScroller = lookup_widget(mainWindow, "treeScroller");
	treeView = lookup_widget(mainWindow, "treeView");
	treePopup = create_treePopup();
	if (treePopup != NULL) {
		treePopupDelete = lookup_widget(treePopup, "treePopupDelete");
		treePopupInsert = lookup_widget(treePopup, "treePopupInsert");
		treePopupRefresh = lookup_widget(treePopup, "treePopupRefresh");
		treePopupSelect = lookup_widget(treePopup, "treePopupSelect");
		treePopupUpdate = lookup_widget(treePopup, "treePopupUpdate");
	}
}
