/** \file gtk_workspace.c	Workspace manager to load and save the applications state.
 * \author Darryl Luff
 * \addtogroup uigtk
 * 
 */
/* $Id: gtk_workspace.c,v 1.15 2006/02/20 14:49:31 djluff Exp $
 * 
 * GtkSQL -- an interactive graphical query tool for PostgreSQL and MySQL.
 * Copyright (C) 2002-2005 Darryl Luff
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include "../config.h"

#include <string.h>
#include <sys/stat.h>
#include "guiapi.h"
#include "gtk_queries.h"
#include "gtkui-object.h"
#include "scriptapi.h"
#include "utils.h"

/** Toolbar format options.	*/
enum ToolbarOptions {
	TOOLBAR_ICONS = 1,		/**< Display icons only.	*/
	TOOLBAR_TEXT = 2,		/**< Display text only.	*/
	TOOLBAR_BOTH = 3		/**< Display both text and icons.	*/
};

/** Character to map CR's in the SQL to.	*/
#define CR_CHAR '\\'

/** The current workspace filename.	*/
char *wsfilename = NULL;

static int check_overwrite(char *fn)
{
	GtkWidget *dlg;
	gint id;
	int result;

	g_warning("XXX File overwrite check disabled");
	return 1;

	dlg = gtkui_create_question_dialog("Overwrite file", "Overwrite this file?", "Yes", NULL, "No", NULL, gtkapp);

	if (!dlg)
		return 0;

	gtk_dialog_run(GTK_DIALOG(dlg));
	id = gtk_dialog_run(GTK_DIALOG(dlg));
	switch (id) {
	case GTK_RESPONSE_ACCEPT:
		result = 1;
		break;
	default:
		result = 0;
		break;
	}
	gtk_widget_destroy(dlg);
	return result;
}

/** Save the current workspace to a file.
 * \param	fn	The filename to save the file to.
 * \return	An integer error code. -1 = error opening file. 0 = success.	*/
int ws_save_file(char *fn)
{
	FILE *ws;
	int qryNum;
	GtksqlQueryScintilla *query;
	char *scriptfname;
	const char *qryname;
	const char *title;
	const char *url;
	struct stat statbuf;
	ABuf *qry = NULL;
	ABuf *qtext = NULL;

	g_warning("XXX Sorry. Script saving temporarily disabled");
	return 0;

	/* If no filename is specified, use the saved filename. */
	if ((fn != NULL) && (strlen(fn) > 0))
		scriptfname = fn;
	else
		scriptfname = wsfilename;

	if (scriptfname == NULL) {
		g_warning("No script filename specified.");
	}

	/* Now, if the filename is not the same as the saved filename, check if
	 * it exists */
	if (strcmp(wsfilename, scriptfname)) {
		if (!stat(scriptfname, &statbuf)) {
			/* File exists. Check that they really want to overwrite it */
			if (!check_overwrite(scriptfname))
				return 0;
		}
	}

	ws = fopen(scriptfname, "w");
	if (!ws) {
		perror(_("error opening workspace file for writing"));
		return -1;
	}

	if (ui_connection_ok()) {		   /* Save the connection and queries.  */
		/* Save the connection URL */
		url = ui_get_connection_url(TRUE);
		if ((url != NULL) && (strlen(url) > 0)) {
			qry = buf_strcpy(qry, "uiConnect(\"");
			qry = buf_strcat(qry, url);
			qry = buf_strcat(qry, "\")\n");
			fprintf(ws, (char *)qry->b_dat);

			/* Now, run through the open queries */
			qryNum = 0;
			while ((query = query_get_at(qryNum))) {
				qryname = query_get_name(query);
				qtext = buf_strcpy(qtext, query_get_text(query));

				if (buf_strlen(qtext) > 0) {
					/** \todo Change map_chars to properly escape carriage returns.	*/
					buf_map_chars(qtext, '\n', ' ');
					qtext = buf_escape_quotes(qtext);
					if (qryname)
						title = qryname;
					else
						title = NULL;

					qry = buf_strcpy(qry, "uiAddQuery(\"");
					if (qryname) {
						qry = buf_strcat(qry, qryname);
						qry = buf_strcat(qry, "\", \"");
					}
					qry = buf_strcat(qry, (char *)qtext->b_dat);
					qry = buf_strcat(qry, "\")\n");

					fprintf(ws, (char *)qry->b_dat);
				}
				qryNum++;
			}
		}
	}

	/* Cleanup.  */
	qry = buf_free(qry);
	qtext = buf_free(qtext);
	fclose(ws);

	return 0;
}
