/***************************************************************************
 *            gtksql_query_scintilla.c
 *
 *  Fri Oct 15 15:05:19 2004
 *  Copyright  2004  Darryl Luff
 *  Email darryl@snakegully.nu
 ****************************************************************************/
/*
 * \todo Font and style setting. Set styles for the three text classes, OR, look at a lexer. Is this easier?
 * \todo Setup syntax highlighting.
 * \todo Setup auto indenting?
 * \todo Setup auto completion.
 * \todo Cut/copy/paste.
 * \todo Look for other scintilla features that can be used.
 *       - macro recording
 *
 * This is a composite widget structured like this:
 * gtksql_query_scintilla(GtkVBox)[3]
 * -query_scintilla_view
 * -query_result_list
 * -hbox[2]
 * --progress_bar
 * --status_bar
 */
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gdk/gdkkeysyms.h>
#include <string.h>

#include "export.h"
#include "gtk_callbacks.h"
#include "gtk_config.h"
#include "gtk_export.h"
#include "gtk_queries.h"
#include "gui_gtk.h"
#include "gtksql_query_scintilla.h"
#include "gtkui-object.h"


/** This should be inside the query widget */
int queryScintillaStatus = 0;

//enum {
//  GTKSQL_QUERY_SIGNAL,
//  LAST_SIGNAL
//};
//static guint gtksql_query_signals[LAST_SIGNAL] = { 0 };

static GtkWidgetClass *parent_class = NULL;
static GtksqlQueryScintillaClass *class = NULL;

static int nextScintillaId = 0;

/* Scintilla style numbers	*/
#define STYLE_SQL	1
#define STYLE_TABLE	2
#define STYLE_FIELD	3

static void gtksql_query_scintilla_class_init(GtksqlQueryScintillaClass *
											  klass);
static void gtksql_query_scintilla_init(GtksqlQueryScintilla * ttt);

const GtkTargetEntry dragTargetsScintilla[] = {
	{"STRING", 0, 0},
	{"text/plain", 0, 1}
};

static void gtksql_query_scintilla_results_display(GtksqlQueryScintilla *
												   query, DBConnection * conn,
												   void *result, int why);

/*
static void gtksql_query_destroy (GtkObject *object) {
	//stop_spinning (GTK_WIDGET (object));
	GTK_OBJECT_CLASS (parent_class)->destroy (object);
}
*/

void gtksql_query_scintilla_get_selection(GtksqlQueryScintilla * query,
										  struct CharacterRange *cr)
{
	if (!query)
		return;

	if (!query->sci)
		return;

	if (!cr)
		return;

	cr->cpMin =
		scintilla_send_message(query->sci, SCI_GETSELECTIONSTART, 0, 0);
	cr->cpMax = scintilla_send_message(query->sci, SCI_GETSELECTIONEND, 0, 0);
}

/**
 * Upon a character being added, SciTE may decide to perform some action
 * such as displaying a completion list or auto-indentation.	*/
void gtksql_query_scintilla_char_added(GtksqlQueryScintilla * query, char ch)
{
	int first;
	DBConnection *conn;
	const Keyword *key = NULL;
	struct TextRange tr;
	ABuf *buf = NULL;

	if (!query) {
		g_warning("gtksql_query_scintilla_char_added() query is NULL\n");
		return;
	}

	tr.chrg.cpMax = scintilla_send_message(query->sci, SCI_GETCURRENTPOS, 0, 0);
	tr.chrg.cpMin = scintilla_send_message(query->sci, SCI_WORDSTARTPOSITION, tr.chrg.cpMax, TRUE);
	if ((tr.chrg.cpMax - tr.chrg.cpMin) < query->minCharsCompletion)
		return;

	tr.lpstrText = malloc((tr.chrg.cpMax - tr.chrg.cpMin) * 2);

	if (!scintilla_send_message(query->sci, SCI_GETTEXTRANGE, 0, (sptr_t) & tr)) {
		g_warning("gtksql_query_scintilla_char_added() couldn't get word\n");
		return;
	}

	/* Build a list of matches */
	conn = ui_get_connection();
	if (!conn)
		return;

	buf = buf_check(buf, 1024);
	buf_strcpy(buf, "");
	key = keyword_match(conn->_keywords, tr.lpstrText);

	first = 1;
	while (key) {
		if (first) {
			buf = buf_strcpy(buf, key->key);
			first = 0;
		}
		else {
			buf = buf_strcat(buf, " ");
			buf = buf_strcat(buf, key->key);
		}
		key = keyword_match_next(key, tr.lpstrText);
	}
	if (buf_strlen(buf) > 0) {
		scintilla_send_message(query->sci, SCI_AUTOCSETIGNORECASE, TRUE, (sptr_t) 0);
		scintilla_send_message(query->sci, SCI_AUTOCSHOW, tr.chrg.cpMax - tr.chrg.cpMin,
			(sptr_t) buf->b_dat);
	}

	buf = buf_free(buf);
}

/*
const GtksqlQueryScintilla *gtksql_query_scintilla_find_id(int id)
{
	int npages;
	int i;
	const GtksqlQueryScintilla *query;
	GtkWidget *queryNotebook = gtkui_get_queryNotebook(gtkapp);

	if (!queryNotebook)
		return NULL;

	npages = gtk_notebook_get_n_pages(GTK_NOTEBOOK(queryNotebook));
	i = 0;
	while (i < npages) {
		query = query_get_at(i);
		if (query != NULL) {
			if (query->sci != NULL) {
				if (scintilla_get_id(query->sci) == id)
					return query;
			}
		}
		i++;
	}
	return NULL;
}
*/

void gtksql_query_scintilla_colourise(GtksqlQueryScintilla * query, int start,
									  int end)
{
	DBConnection *conn;
	const Keyword *key;
	struct TextRange tr;
	int pos;

	if (!query)
		return;

	if (start == end)
		return;

	/* Need to move from 'start' to 'end', colourising each word. */
	pos = start;
	tr.chrg.cpMin = pos;			   // SSM(SCI_WORDENDPOSITION, pos,
									   // TRUE);
	tr.chrg.cpMax = scintilla_send_message(query->sci, SCI_WORDENDPOSITION, pos, TRUE);
	if (tr.chrg.cpMax == end)
		return;

	if (tr.chrg.cpMax > tr.chrg.cpMin) {	/* Real word */
		tr.lpstrText = g_malloc((tr.chrg.cpMax - tr.chrg.cpMin) * 2);
		if (tr.lpstrText) {
			/* get the word */
			if (scintilla_send_message(query->sci, SCI_GETTEXTRANGE, 0, (sptr_t) & tr) > 0) {
				/* Check the word against the lists */
				conn = ui_get_connection();
				if (conn) {
					key = keyword_match_exact(conn->_keywords, tr.lpstrText);
					if (key) {
						scintilla_send_message(query->sci, SCI_STARTSTYLING, tr.chrg.cpMin, 0x1f);
						switch (key->type) {
						case KEYWORD_FREE:
							break;
						case KEYWORD_SQL:
							scintilla_send_message(query->sci, SCI_SETSTYLING, tr.chrg.cpMax - tr.chrg.cpMin,
								STYLE_SQL);
							break;
						case KEYWORD_TABLE:
							scintilla_send_message(query->sci, SCI_SETSTYLING, tr.chrg.cpMax - tr.chrg.cpMin,
								STYLE_TABLE);
							break;
						case KEYWORD_FIELD:
							scintilla_send_message(query->sci, SCI_SETSTYLING, tr.chrg.cpMax - tr.chrg.cpMin,
								STYLE_FIELD);
							break;
						}
					}
				}
			}
			g_free(tr.lpstrText);
			tr.lpstrText = NULL;
		}
	}
	else {							   /* Check if we're at the start of some 
									    * space. */
		tr.chrg.cpMin = pos;		   // SSM(SCI_WORDENDPOSITION, pos,
									   // TRUE);
		tr.chrg.cpMax = scintilla_send_message(query->sci, SCI_WORDENDPOSITION, pos, FALSE);
		if (tr.chrg.cpMax > tr.chrg.cpMin) {
			tr.lpstrText = g_malloc((tr.chrg.cpMax - tr.chrg.cpMin) * 2);
			if (tr.lpstrText) {
				/* get the word */
				if (scintilla_send_message(query->sci, SCI_GETTEXTRANGE, 0, (sptr_t) & tr) > 0) {
					scintilla_send_message(query->sci, SCI_STARTSTYLING, tr.chrg.cpMin, 0x1f);
					scintilla_send_message(query->sci, SCI_SETSTYLING, tr.chrg.cpMax - tr.chrg.cpMin, STYLE_DEFAULT);
				}
				g_free(tr.lpstrText);
				tr.lpstrText = NULL;
			}
		}
	}
}

void gtksql_query_scintilla_notify(GtkWidget * editor, gint scn,
								   struct SCNotification *notification,
								   GtksqlQueryScintilla * query)
{
	long lstart;
	long lend;
	struct TextRange tr;
	char chr[2];

	if (!query)
		return;

	switch (notification->nmhdr.code) {
	case SCN_STYLENEEDED:
		/* Need to apply styling from ENDSTYLED to the position in the
		 * notification */
		gtksql_query_scintilla_colourise(query, scintilla_send_message(query->sci, SCI_GETENDSTYLED, 0, 0),
										 notification->position);
		break;

	case SCN_CHARADDED:
		gtksql_query_scintilla_char_added(query, notification->ch);
		break;

	case SCN_UPDATEUI:
		/* do brace matching	*/
		lstart = scintilla_send_message(query->sci, SCI_GETCURRENTPOS, 0, 0);

		tr.chrg.cpMin = lstart - 1;
		tr.chrg.cpMax = lstart;
		tr.lpstrText = (char *)&chr;

		if (scintilla_send_message(query->sci, SCI_GETTEXTRANGE, 0, (sptr_t)&tr) > 0) {
			if ((chr[0] == '(') || (chr[0] == ')')) {
				lend = scintilla_send_message(query->sci, SCI_BRACEMATCH, lstart-1, 0);

				/* if there is a matching brace highlight it	*/
				if (lend >= 0)
					scintilla_send_message(query->sci, SCI_BRACEHIGHLIGHT, lstart-1, lend);
				else
					scintilla_send_message(query->sci, SCI_BRACEBADLIGHT, lstart-1, 0);
			}
			else	{ /* if there is NO matching brace erase old highlight	*/
				// scintilla_send_message(query->sci, SCI_BRACEHIGHLIGHT, -1, -1);
			}
		}
		//else
		//	scintilla_send_message(query->sci, SCI_BRACEHIGHLIGHT, -1, -1);

		break;
   }
}

static void gtksql_query_scintilla_on_finalize(GObject * object)
{
//  GtkCalendarPrivateData *private_data;
//  private_data = GTK_CALENDAR_PRIVATE_DATA (object);
	// g_free (private_data);
	if (G_OBJECT_CLASS(parent_class)->finalize)
		(*G_OBJECT_CLASS(parent_class)->finalize) (object);
}

void *gtksql_query_scintilla_get_result(GtksqlQueryScintilla * query)
{
	if (!query)
		return NULL;

	return query->result;
}

static char *txtBuf = NULL;
const gchar *gtksql_query_scintilla_get_text(GtksqlQueryScintilla * query)
{
	int len;

	if (!query)
		return "";

	if (!query->sci)
		return "";

	if (txtBuf)
		g_free(txtBuf);

	/* Have to add 1 to length to make room for NULL	*/
	len = scintilla_send_message(query->sci, SCI_GETLENGTH, 0, 0) + 1;

	if (len < 2)
		return "";

	txtBuf = malloc(len);
	if (!txtBuf)
		return "";

	scintilla_send_message(query->sci, SCI_GETTEXT, len, (sptr_t) txtBuf);
	return txtBuf;
}

GType gtksql_query_scintilla_get_type(void)
{
	static GType ttt_type = 0;

	if (!ttt_type) {
		static const GTypeInfo ttt_info = {
			sizeof(GtksqlQueryScintillaClass),
			NULL,					   /* base_init */
			NULL,					   /* base_finalize */
			(GClassInitFunc) gtksql_query_scintilla_class_init,
			NULL,					   /* class_finalize */
			NULL,					   /* class_data */
			sizeof(GtksqlQueryScintilla),
			0,
			(GInstanceInitFunc) gtksql_query_scintilla_init,
		};

		ttt_type =
			g_type_register_static(GTK_TYPE_VBOX, "GtksqlQueryScintilla",
								   &ttt_info, 0);
	}
	return ttt_type;
}

static void gtksql_query_scintilla_on_unrealize(GtkWidget * widget)
{
	GtksqlQueryScintilla *query;

	query = GTKSQL_QUERY_SCINTILLA(widget);
	if (query->query_name) {
		g_free(query->query_name);
		query->query_name = NULL;
	}
	if (GTK_WIDGET_CLASS(parent_class)->unrealize)
		(*GTK_WIDGET_CLASS(parent_class)->unrealize) (widget);
}

int scintilla_colour_from_gdk_color(GdkColor color)
{
	/* GdkColor rgb values are 16 bit, and scintilla uses 8 bit values
	 * arranged like: red | (green << 8) | (blue << 16) */
	return (int) (color.red >> 8) | (color.green & 0xff00) | ((color.blue & 0xff00) << 8);
}

void gtksql_query_scintilla_class_set_field_colour(const gchar * cvalue)
{
	GdkColor colour;

	if (!class)
		return;

	if (gdk_color_parse(cvalue, &colour))
		class->fieldColour = scintilla_colour_from_gdk_color(colour);
}

void gtksql_query_scintilla_class_set_normal_colour(const gchar * cvalue)
{
	GdkColor colour;

	if (!class)
		return;

	if (gdk_color_parse(cvalue, &colour))
		class->normalColour = scintilla_colour_from_gdk_color(colour);
}

void gtksql_query_scintilla_class_set_font_name(const gchar * fname)
{
	if (!class)
		return;

	if (class->fontName) {
		g_free(class->fontName);
		class->fontName = NULL;
	}

	if (fname)
		class->fontName = g_strdup(fname);
}

void gtksql_query_scintilla_class_set_sql_colour(const gchar * cvalue)
{
	GdkColor colour;

	if (!class)
		return;

	if (gdk_color_parse(cvalue, &colour))
		class->sqlColour = scintilla_colour_from_gdk_color(colour);
}

void gtksql_query_scintilla_class_set_table_colour(const gchar * cvalue)
{
	GdkColor colour;

	if (!class)
		return;

	if (gdk_color_parse(cvalue, &colour))
		class->tableColour = scintilla_colour_from_gdk_color(colour);
}

static void setup_tags()
{
	if (!class)
		return;

	gtksql_query_scintilla_class_set_normal_colour(gtkcfg_get_query_normal_colour());
	gtksql_query_scintilla_class_set_field_colour(gtkcfg_get_query_field_colour());
	gtksql_query_scintilla_class_set_sql_colour(gtkcfg_get_query_sql_colour());
	gtksql_query_scintilla_class_set_table_colour(gtkcfg_get_query_table_colour());
	gtksql_query_scintilla_class_set_font_name(gtkcfg_get_query_font_name());
}

static void gtksql_query_scintilla_class_init(GtksqlQueryScintillaClass *
											  klass)
{
	GObjectClass *gobject_class;
	GtkObjectClass *object_class;
	GtkWidgetClass *widget_class;

	gobject_class = (GObjectClass *) klass;
	object_class = (GtkObjectClass *) klass;
	widget_class = (GtkWidgetClass *) klass;

	parent_class = g_type_class_peek_parent(klass);

	/* global class pointer */
	class = (GtksqlQueryScintillaClass *) klass;

	// object_class->destroy = gtksql_query_destroy;
	gobject_class->finalize = gtksql_query_scintilla_on_finalize;
	widget_class->unrealize = gtksql_query_scintilla_on_unrealize;

	/* Class data */
	setup_tags();
}

void gtksql_query_scintilla_set_result_widget(GtksqlQueryScintilla * query,
											  GtkWidget * widget)
{
	if (!query) {
		g_warning
			("gtksql_query_scintilla_set_result_widget(): query is NULL\n");
		return;
	}

	/* Remove existing widget */
	if (query->result_widget) {
		gtk_container_remove(GTK_CONTAINER(query->vpane),
							 query->result_widget);
		gtk_widget_unref(query->result_widget);
	}

	if (widget)
		query->result_widget = widget;
	else
		query->result_widget = gtk_label_new("");

	gtk_widget_ref(query->result_widget);
	gtk_paned_add2(GTK_PANED(query->vpane), query->result_widget);
}

void gtksql_query_scintilla_set_styles(GtksqlQueryScintilla * query)
{
	ABuf *buf = buf_new(100);
	PangoFontDescription *pfd;
	gint size;
	gint italic;

	if (!query) {
		g_warning("gtksql_query_scintilla_set_styles(%p): query NULL\n", query);
		return;
	}

	if (!query->sci) {
		g_warning("gtksql_query_scintilla_set_styles(%p): query->sci NULL\n", query);
		return;
	}

	scintilla_send_message(query->sci, SCI_STYLESETFORE, STYLE_DEFAULT, class->normalColour);
	if (class->fontName) {
	
		pfd = pango_font_description_from_string(class->fontName);
		if (pfd) {
			buf = buf_strcpy(buf, "!");	/* use pango	*/
			buf = buf_strcat(buf, pango_font_description_get_family(pfd));
			size = pango_font_description_get_size(pfd);
			if (! pango_font_description_get_size_is_absolute (pfd))
				size = size / PANGO_SCALE;

			italic = (pango_font_description_get_style(pfd) == PANGO_STYLE_ITALIC);
			scintilla_send_message(query->sci, SCI_STYLESETFONT, STYLE_DEFAULT, (sptr_t) buf->b_dat);
			scintilla_send_message(query->sci, SCI_STYLESETITALIC, STYLE_DEFAULT, italic);
			scintilla_send_message(query->sci, SCI_STYLESETSIZE, STYLE_DEFAULT, size);
			switch (pango_font_description_get_weight(pfd)) {
				case PANGO_WEIGHT_ULTRALIGHT:
				case PANGO_WEIGHT_LIGHT:
				case PANGO_WEIGHT_NORMAL:
					scintilla_send_message(query->sci, SCI_STYLESETBOLD, STYLE_DEFAULT, 0);
					break;
				
				case PANGO_WEIGHT_SEMIBOLD:
				case PANGO_WEIGHT_BOLD:
				case PANGO_WEIGHT_ULTRABOLD:
				case PANGO_WEIGHT_HEAVY:
					scintilla_send_message(query->sci, SCI_STYLESETBOLD, STYLE_DEFAULT, 1);
					break;
			}
		}
	}

	scintilla_send_message(query->sci, SCI_STYLECLEARALL, 0, 0);
	scintilla_send_message(query->sci, SCI_STYLESETFORE, STYLE_SQL, class->sqlColour);
	scintilla_send_message(query->sci, SCI_STYLESETFORE, STYLE_TABLE, class->tableColour);
	scintilla_send_message(query->sci, SCI_STYLESETFORE, STYLE_FIELD, class->fieldColour);

	/* Set the default styles for brace highlights	*/
	/* Bold blue	*/
	scintilla_send_message(query->sci, SCI_STYLESETFORE, STYLE_BRACELIGHT, 255 << 16);
	scintilla_send_message(query->sci, SCI_STYLESETBOLD, STYLE_BRACELIGHT, 1);

	/* Bold red	*/
	scintilla_send_message(query->sci, SCI_STYLESETFORE, STYLE_BRACEBAD, 255);
	scintilla_send_message(query->sci, SCI_STYLESETBOLD, STYLE_BRACEBAD, 1);
	//scintilla_send_message(query->sci, SCI_STYLESETBACK, STYLE_BRACEBAD, 128 || 128 << 8);

	buf = buf_free(buf);
}

static void gtksql_query_scintilla_clear(GtksqlQueryScintilla * query, DBConnection * conn)
{
	if (!query) {
		g_warning("gtksql_query_scintilla_clear(%p): query NULL\n", query);
		return;
	}

	if (!query->sci) {
		g_warning("gtksql_query_scintilla_clear(%p): query->sci NULL\n", query);
		return;
	}

	scintilla_send_message(query->sci, SCI_CLEARALL, 0, 0);

	if (conn)
		gtksql_query_scintilla_results_display(query, conn, NULL, QUERY_FAKE);
}

void gtksql_query_scintilla_init_view(GtksqlQueryScintilla * query)
{
	if (!query)
		return;

	if (!query->editor)
		query->editor = scintilla_new();

	if (!query->sci) {
		query->sci = SCINTILLA(query->editor);
		scintilla_set_id(query->sci, nextScintillaId++);
	}
	gtk_widget_set_size_request(query->editor, 150, 100);

	scintilla_send_message(query->sci, SCI_STYLECLEARALL, 0, 0);
	scintilla_send_message(query->sci, SCI_SETLEXER, SCLEX_CONTAINER, 0);
	scintilla_send_message(query->sci, SCI_SETINDENT, 2, 0);
	scintilla_send_message(query->sci, SCI_SETINDENTATIONGUIDES, 1, 0);
	gtksql_query_scintilla_clear(query, NULL);
	gtksql_query_scintilla_set_styles(query);
}

void gtksql_query_scintilla_append_text(GtksqlQueryScintilla * query,
										gchar * s)
{
	if (!query)
		return;

	if (!s)
		return;

	scintilla_send_message(query->sci, SCI_APPENDTEXT, strlen(s), (sptr_t) s);
}

/** Event handler for key presses. Handle all the hot-keys.
 * \param widget	The sending widget.
 * \param event	The keypress event.
 * \param data	The gtkui object.	*/
static gint
gtksql_query_scintilla_key_press_event(GtkWidget * widget, GdkEventKey * event, gpointer data)
{
	GtkNotebook *notebook = NULL;

	if (event->state == GDK_MOD1_MASK) {
		// g_signal_stop_emission_by_name(G_OBJECT(data), "key_press_event");

		notebook = GTK_NOTEBOOK(gtkui_get_queryNotebook(gtkapp));

		if (event->keyval == 'n') {
			query_add();
			return (TRUE);
		}

		if (event->keyval == 'd') {
			if (query_count() > 1) {
				query_close();
			}
			return (TRUE);
		}

		if (event->keyval == 's') {
			ui_send_query();
			return (TRUE);
		}

		if (event->keyval == 'e') {
			do_export_dialog();
			return (TRUE);
		}

		if (event->keyval == GDK_Left) {
			int curpage;

			gtk_notebook_prev_page(notebook);
			curpage = gtk_notebook_get_current_page(notebook);
			gtk_widget_grab_focus(gtk_notebook_get_nth_page(notebook, curpage));

			return (TRUE);
		}

		if (event->keyval == GDK_Right) {
			int curpage;

			gtk_notebook_next_page(notebook);

			curpage = gtk_notebook_get_current_page(notebook);
			gtk_widget_grab_focus(gtk_notebook_get_nth_page(notebook, curpage));

			return (TRUE);
		}
	}
	return (FALSE);
}

/* 'sci' can be either a 'scintilla', or a 'gtksql_query_scintilla'. Check for both.	*/
void gtksql_query_scintilla_do_copy(GtkWidget *sc) {
	ScintillaObject *sci = NULL;

	if (IS_SCINTILLA(sc))
		sci = (ScintillaObject *)sc;
	else if (IS_GTKSQL_QUERY_SCINTILLA(sc))
		sci = GTKSQL_QUERY_SCINTILLA(sc)->sci;

	if (sci)
		scintilla_send_message(sci, SCI_COPY, 0, 0);
}

/* 'sci' can be either a 'scintilla', or a 'gtksql_query_scintilla'. Check for both.	*/
void gtksql_query_scintilla_do_cut(GtkWidget *sc) {
	ScintillaObject *sci = NULL;

	if (IS_SCINTILLA(sc))
		sci = (ScintillaObject *)sc;
	else if (IS_GTKSQL_QUERY_SCINTILLA(sc))
		sci = GTKSQL_QUERY_SCINTILLA(sc)->sci;

	if (sci)
		scintilla_send_message(sci, SCI_CUT, 0, 0);
}

/* 'sci' can be either a 'scintilla', or a 'gtksql_query_scintilla'. Check for both.	*/
void gtksql_query_scintilla_do_paste(GtkWidget *sc) {
	ScintillaObject *sci = NULL;

	if (IS_SCINTILLA(sc))
		sci = (ScintillaObject *)sc;
	else if (IS_GTKSQL_QUERY_SCINTILLA(sc))
		sci = GTKSQL_QUERY_SCINTILLA(sc)->sci;

	if (sci)
		scintilla_send_message(sci, SCI_PASTE, 0, 0);
}

static void editor_size_allocate_callback(GtkWidget *widget, GtkAllocation *allocation, gpointer user_data) {
	GtksqlQueryScintilla *query;

	query = GTKSQL_QUERY_SCINTILLA(user_data);
	if (! query)
		return;

	if (! query->sci)
		return;

	/* This shouldn't work, because setting the scroll width to the width of the 
	 * widget should mean that it never scrolls? But it seems to work OK anyway.	*/
	scintilla_send_message(query->sci, SCI_SETSCROLLWIDTH, allocation->width, 0);
}

/** Create the child widgets and initialise the instance variables to default values.	*/
static void gtksql_query_scintilla_init(GtksqlQueryScintilla * query)
{
	query->query_name = g_strdup("Query");
	query->editor = NULL;
	query->sci = NULL;
	query->vpane = NULL;
	query->progress_bar = NULL;
	query->status_bar = NULL;
	query->result = NULL;
	query->queryInProgress = 0;
	query->minCharsCompletion = 2;

	/* The containing vbox */
	GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
	if (!vbox) {
		g_warning("Could not allocate vbox\n");
		return;
	}

	query->vpane = gtk_vpaned_new();

	/* Add the text field to the scroller */
	if (!class) {
		class = GTKSQL_QUERY_SCINTILLA_GET_CLASS(query);
		g_object_ref(G_OBJECT(class));
	}

	gtksql_query_scintilla_init_view(query);
	gtk_widget_ref(query->editor);
	g_signal_connect(G_OBJECT(query->editor), "size-allocate",
					 G_CALLBACK(editor_size_allocate_callback), query);

	gtk_paned_add1(GTK_PANED(query->vpane), query->editor);
	g_signal_connect(G_OBJECT(query->editor), "sci-notify",
					 G_CALLBACK(gtksql_query_scintilla_notify), query);

	/* fake for now */
	gtksql_query_scintilla_set_result_widget(query, NULL);
	gtk_box_pack_start(GTK_BOX(vbox), query->vpane, TRUE, TRUE, 0);

	/* Progress and status bars */
	GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
	query->progress_bar = gtk_progress_bar_new();
	gtk_widget_ref(query->progress_bar);
	gtk_box_pack_start(GTK_BOX(hbox), query->progress_bar, FALSE, FALSE, 0);

	query->status_bar = gtk_statusbar_new();
	gtk_widget_ref(query->status_bar);
	query->statusId =
		gtk_statusbar_get_context_id(GTK_STATUSBAR(query->status_bar),
									 "Query status");
	gtk_box_pack_start(GTK_BOX(hbox), query->status_bar, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

	gtk_widget_show(vbox);
	gtk_widget_show(query->editor);
	gtk_widget_show(query->vpane);
	gtk_widget_show(hbox);
	gtk_widget_show(query->progress_bar);
	gtk_widget_show(query->status_bar);

	gtk_box_pack_start(GTK_BOX(query), vbox, TRUE, TRUE, 0);
	g_signal_connect(G_OBJECT(query), "key_press_event", G_CALLBACK(gtksql_query_scintilla_key_press_event), gtkapp);

	gtk_drag_dest_set(GTK_WIDGET(query), GTK_DEST_DEFAULT_ALL, dragTargetsScintilla, DRAG_TARGET_COUNT, GDK_ACTION_COPY);
	// g_signal_connect(G_OBJECT(query->query_text_view), "populate_popup",
	// G_CALLBACK(query_append_popup), query->query_text_view);

	gtksql_query_scintilla_set_styles(query);
}

/** Displays a fake result for a non-existant query.	*/
static GtkWidget *gtksql_query_scintilla_results_display_fake()
{
	GtkWidget *resultWidget;

	resultWidget = gtk_label_new("No query.");
	gtk_widget_show(resultWidget);
	return resultWidget;
}

GtkWidget *gtksql_query_scintilla_new(void)
{
	return GTK_WIDGET(g_object_new(gtksql_query_scintilla_get_type(), NULL));
}

/** Display an error message as a result.	*/
static GtkWidget *gtksql_query_scintilla_results_display_error(void *result)
{
	GtkWidget *resultWidget;
	if (result == NULL)
		resultWidget = gtk_label_new("Error in query.");
	else
		resultWidget = gtk_label_new((char *) result);

	gtk_label_set_line_wrap(GTK_LABEL(resultWidget ), TRUE);
	gtk_label_set_single_line_mode(GTK_LABEL(resultWidget ), FALSE);

	return resultWidget;
}

static void gtksql_query_scintilla_status_push(GtksqlQueryScintilla * query,
											   gchar * status)
{
	if (!query)
		return;

	if (!query->status_bar)
		return;

	/** Push a new message onto the status bar stack.	*/
	gtk_statusbar_push(GTK_STATUSBAR(query->status_bar), query->statusId,
					   status);
	while (gtk_events_pending())
		gtk_main_iteration();
}

void gtksql_query_scintilla_undo(GtksqlQueryScintilla * query)
{
	if (!query)
		return;

	if (!query->sci)
		return;

	scintilla_send_message(query->sci, SCI_UNDO, 0, (sptr_t) NULL);
}

/** Popup result menu on button click.
 * \param widget	The button.
 * \param event	The button event.
 * \param user_data	Not used.	*/
static gboolean gtksql_query_scintilla_on_result_button_press_event(GtkWidget
																	* widget,
																	GdkEventButton
																	* event,
																	gpointer
																	user_data)
{
	GtkWidget *resultPopup = NULL;

	if (event->button == 3) {
		resultPopup = gtkui_get_resultPopup(gtkapp);
		if (resultPopup == NULL) {
			g_warning("resultPopup not initialised\n");
			return FALSE;
		}
		gtk_menu_popup(GTK_MENU(resultPopup), NULL, NULL, NULL, NULL,
					   event->button, event->time);
		return TRUE;
	}
	return FALSE;
}

/** Popup the result menu.	*/
static void gtksql_query_scintilla_results_popup_menu(GtkWidget * treeview,
													  GdkEventButton * event,
													  gpointer userdata)
{
	GtkWidget *resultPopup = gtkui_get_resultPopup(gtkapp);

	if (!resultPopup)
		return;

	/* Note: event can be NULL here when called from view_onPopupMenu;
	 * gdk_event_get_time() accepts a NULL argument */
	gtk_menu_popup(GTK_MENU(resultPopup), NULL, NULL, NULL, NULL,
				   (event != NULL) ? event->button : 0,
				   gdk_event_get_time((GdkEvent *) event));
}

/** Popup the result menu.	*/
static gboolean gtksql_query_scintilla_on_result_popup_menu(GtkWidget *
															treeview,
															gpointer userdata)
{
	gtksql_query_scintilla_results_popup_menu(treeview, NULL, userdata);
	return TRUE;
}

/** Display the result for a new query.	*/
static GtkWidget
	* gtksql_query_scintilla_results_display_new(GtksqlQueryScintilla * query,
												 DBConnection * conn,
												 void *result)
{
	const char **cols;
	GtkWidget *resultWidget;
	int stat;
	int nb_tuples;
	const char **col_titles;
	int nb_cols;
	GType *columns;
	GtkListStore *model;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	int i;
	int j;
	GtkTreeIter listiter;
	GtkWidget *view = NULL;

	gtksql_query_scintilla_status_push(query, "Displaying query results...");
	gtksql_query_scintilla_progress_reset(query);

	stat = dbconnection_get_query_status(conn, result);
	if (stat == DB_QUERY_RECORDS_OK) {
		nb_tuples = dbconnection_get_record_count(conn, result);
		nb_cols = dbconnection_get_field_count(conn, result);

		/* Create the scroller */
		resultWidget = gtk_scrolled_window_new(NULL, NULL);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
									   (resultWidget),
									   GTK_POLICY_AUTOMATIC,
									   GTK_POLICY_AUTOMATIC);

		/* Init the List */
		col_titles = (const char **) g_malloc(nb_cols * sizeof(char *));
		columns = g_malloc(sizeof(GType) * nb_cols);
		for (i = 0; i < nb_cols; i++) {
			col_titles[i] = dbconnection_get_field_name(conn, result, i);
			columns[i] = G_TYPE_STRING;
		}

		/* Model initialisation */
		model = gtk_list_store_newv(nb_cols, columns);
		if (model == NULL) {
			/* No need for warning. This occurs normally sometimes. */
			return NULL;
		}

		view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(model));
		if (view == NULL) {
			g_warning("results_display_query_new() view is NULL\n");
			return NULL;
		}
		gtk_tree_view_set_reorderable(GTK_TREE_VIEW(view), TRUE);

		g_free(col_titles);
		col_titles = NULL;
		g_free(columns);
		columns = NULL;

		/* Add the columns */
		for (i = 0; i < nb_cols; i++) {
			renderer = gtk_cell_renderer_text_new();
			column = gtk_tree_view_column_new_with_attributes
				(dbconnection_get_field_name(conn, result, i),
				 renderer, "text", i, NULL);
			gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
		}
		gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(view), TRUE);
		gtk_widget_show(GTK_WIDGET(view));
		gtk_container_add(GTK_CONTAINER(resultWidget), view);

		/* Load the data */
		cols = g_malloc(sizeof(char *) * nb_cols);
		if (cols == NULL) {
			g_warning("results_display_query_new() cols is NULL\n");
			return NULL;
		}
		for (i = 0; i < nb_tuples; i++) {
			gtksql_query_scintilla_progress_update(query,
												   (gfloat) i / nb_tuples);
			gtk_list_store_append(model, &listiter);
			for (j = 0; j < nb_cols; j++) {
				cols[j] = dbconnection_get_field_value(conn, result, i, j);

				/* take care of null fields */
				if (cols[j] == NULL)
					cols[j] = "";

				gtk_list_store_set(model, &listiter, j, cols[j], -1);
			}
		}
		g_free(cols);
		cols = NULL;
		g_free(columns);
		columns = NULL;

		g_signal_connect(G_OBJECT(view), "button_press_event",
						 G_CALLBACK
						 (gtksql_query_scintilla_on_result_button_press_event),
						 view);
		g_signal_connect(G_OBJECT(view), "popup-menu",
						 G_CALLBACK
						 (gtksql_query_scintilla_on_result_popup_menu), view);

		gtk_widget_show(resultWidget);
		gtk_tree_view_columns_autosize(GTK_TREE_VIEW(view));
	}
	else if (dbconnection_get_query_status(conn, result) ==
			 DB_QUERY_COMMAND_OK)
		resultWidget = gtk_label_new("Command ok.");
	else if (dbconnection_get_query_status(conn, result) == DB_QUERY_EMPTY)
		resultWidget = gtk_label_new("Empty query.");
	else
		resultWidget = gtk_label_new("Error in query.");

	gtksql_query_scintilla_status_pop(query);

	return resultWidget;
}

void gtksql_query_scintilla_redo(GtksqlQueryScintilla * query)
{
	if (!query)
		return;

	if (!query->sci)
		return;

	scintilla_send_message(query->sci, SCI_REDO, 0, (sptr_t) NULL);
}

/** Display the results in the result pane.
 * \param result	Depends on 'why' param. For good queries, points to the database result structure.
 * For QUERY_ERROR, is an error message.
 * \param why	Why are the results being displayed? Can be:
 *  - QUERY_FAKE	A new label is set as the current page result widget.
 *  - QUERY_SWITCH	The result notepad is switched to the appropriate page.
 *  - QUERY_ERROR	A new label is set as the current page result widget.
 *  - Otherwise, the result widget is set to a GtkTreeView containing the query results inside a scrolled window.
 * \param position	The result notebook page number to set the result on. If < 0,
 * it is switched to the page corresponding to the current query.	*/
static void gtksql_query_scintilla_results_display(GtksqlQueryScintilla *
												   query, DBConnection * conn,
												   void *result, int why)
{
	GtkWidget *resultWidget = NULL;

	if (why == QUERY_FAKE) {		   /* Add a dummy message */
		resultWidget = gtksql_query_scintilla_results_display_fake();
	}
	else {							   /* Both ERROR and NEW need some extra
									    * processing. */
		if (why == QUERY_ERROR) {
			resultWidget =
				gtksql_query_scintilla_results_display_error(result);
		}
		else if (why == QUERY_NEW) {
			resultWidget =
				gtksql_query_scintilla_results_display_new(query, conn,
														   result);
		}
		if (resultWidget) {
			gtk_widget_show(resultWidget);
			gtksql_query_scintilla_set_result_widget(query, resultWidget);
		}
	}
}

void gtksql_query_scintilla_free(GtksqlQueryScintilla * query)
{
	gtksql_query_scintilla_clear(query, ui_get_connection());

	if (query->query_name) {
		g_free(query->query_name);
		query->query_name = NULL;
	}

	if (query->editor) {
		gtk_widget_unref(query->editor);
		query->editor = NULL;
	}

	if (query->vpane) {
		gtk_widget_unref(query->vpane);
		query->vpane = NULL;
	}

	if (query->result_widget) {
		gtk_widget_unref(query->result_widget);
		query->result_widget = NULL;
	}

	if (query->progress_bar) {
		gtk_widget_unref(query->progress_bar);
		query->progress_bar = NULL;
	}

	if (query->status_bar) {
		gtk_widget_unref(query->status_bar);
		query->status_bar = NULL;
	}
}

const gchar *gtksql_query_scintilla_get_name(GtksqlQueryScintilla * ttt)
{
	if (!ttt)
		return NULL;

	return ttt->query_name;
}

/** This needs to destroy any widgets owned by us that arent child widgets.
 * So nothing yet, but maybe should destroy the result popup here? */
/*
static void gtksql_query_scintilla_on_destroy(GtkObject *object, gpointer user_data) {
}
*/

/** Reset the progress bar.	*/
void gtksql_query_scintilla_progress_reset(GtksqlQueryScintilla * query)
{
	if (!query)
		return;

	if (!query->progress_bar)
		return;

	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(query->progress_bar), 0.0);
	while (gtk_events_pending())
		gtk_main_iteration();
}

/** Update the progress bar.	*/
void gtksql_query_scintilla_progress_update(GtksqlQueryScintilla * query,
											gfloat percentage)
{
	if (!query)
		return;

	if (!query->progress_bar)
		return;

	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(query->progress_bar),
								  percentage);
	while (gtk_events_pending())
		gtk_main_iteration();
}

/** Send the query to the database.	*/
void gtksql_query_scintilla_send(GtksqlQueryScintilla * query,
								 DBConnection * conn)
{
#define ERROREXEC_S	" Error executing query \n %s "
	char status[1024];
	const gchar *cquery;
	void *result;
	const char *msg;
	ABuf *buf = NULL;

	status[0] = '\0';

	if (!query)
		return;

	cquery = gtksql_query_scintilla_get_text(query);

	if (query->result != NULL) {
		dbconnection_clear_result(conn, query->result);
		query->result = NULL;
	}

	/* Clear old error message. */
	gtksql_query_scintilla_results_display(query, conn, NULL, QUERY_FAKE);
	gtksql_query_scintilla_status_push(query, "Query in progress...");

	query->queryInProgress++;
	update_ui();

	gtksql_query_scintilla_progress_reset(query);
	result = dbconnection_execute_query(conn, cquery);
	gtksql_query_scintilla_progress_reset(query);

	query->result = result;
	if ((dbconnection_get_query_status(conn, result) != DB_QUERY_RECORDS_OK)
		&& (dbconnection_get_query_status(conn, result) !=
			DB_QUERY_COMMAND_OK)
		&& (dbconnection_get_query_status(conn, result) != DB_QUERY_EMPTY)) {
		msg = dbconnection_get_error_message(conn);
		if (msg == NULL)
			msg = "";

		buf = buf_new(strlen(ERROREXEC_S) + strlen(msg));
		if (!buf) {
			g_warning("SendQuery() could not allocate memory for a buffer\n");
			query->queryInProgress--;
			update_ui();
			return;
		}
		snprintf((char *)buf->b_dat, buf->b_len, ERROREXEC_S, msg);
		ui_show_error("Error executing query", (char *)buf->b_dat);
		gtksql_query_scintilla_results_display(query, conn,
											   (void *) buf->b_dat,
											   QUERY_ERROR);

		strncpy(status, "Error in query", sizeof(status));
		buf_free(buf);
	}
	else {
		int nbf = dbconnection_get_field_count(conn, result);
		int nbr = dbconnection_get_record_count(conn, result);

		gtksql_query_scintilla_results_display(query, conn, result,
											   QUERY_NEW);
		// sprintf(status, "Query results : %d field%sand %d record%s", nbf, (nbf != 1 ? "s " : " "), nbr, (nbr != 1 ? "s" : ""));
		snprintf(status, sizeof(status)-1, "Query results : %d field%sand %d record%s", nbf, (nbf != 1 ? "s " : " "), nbr, (nbr != 1 ? "s" : ""));
		status[sizeof(status)-1] = '\0';
	}

	query->queryInProgress--;
	update_ui();
	gtksql_query_scintilla_progress_reset(query);
	gtksql_query_scintilla_status_pop(query);

	if (strlen(status) > 0) {
		if (queryScintillaStatus == 1)
			gtksql_query_scintilla_status_pop(query);
		gtksql_query_scintilla_status_push(query, status);
		queryScintillaStatus = 1;
	}
	else
		queryScintillaStatus = 0;
}

void gtksql_query_scintilla_set_name(GtksqlQueryScintilla * ttt,
									 const gchar * name)
{
	if (!ttt)
		return;

	if (ttt->query_name)
		g_free(ttt->query_name);

	if (name == NULL)
		ttt->query_name = NULL;
	else
		ttt->query_name = g_strdup(name);
}

void gtksql_query_scintilla_set_text(GtksqlQueryScintilla * query,
									 const gchar * txt)
{
	if (!query)
		return;

	if (!query->sci)
		return;

	if (txt)
		scintilla_send_message(query->sci, SCI_SETTEXT, 0, (sptr_t) txt);
	else
		scintilla_send_message(query->sci, SCI_SETTEXT, 0, (sptr_t) "");
}

void gtksql_query_scintilla_set_save_point(GtksqlQueryScintilla * query)
{
	if (!query)
		return;

	if (!query->sci)
		return;

	scintilla_send_message(query->sci, SCI_SETSAVEPOINT, 0, 0);
}

void gtksql_query_scintilla_status_pop(GtksqlQueryScintilla * query)
{
	if (!query)
		return;

	if (!query->status_bar)
		return;

	/** Push a new message onto the status bar stack.	*/
	gtk_statusbar_pop(GTK_STATUSBAR(query->status_bar), query->statusId);
	while (gtk_events_pending())
		gtk_main_iteration();
}
