/** \file gtkui-object.c	Experiment with encapsulating global variables in a gobject.
 * \author Darryl Luff
 * \addtogroup uigtk
 * 
 */
/* $Id: gtkui-object.c,v 1.13 2007/12/01 12:00:12 djluff Exp $
 * 
 * GtkSQL -- an interactive graphical query tool for PostgreSQL and MySQL.
 * Copyright (C) 2002-2006 Darryl Luff
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include "../config.h"
#include <sys/utsname.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gdk/gdkkeysyms.h>
#include <gdk/gdkx.h>
#include <gtk/gtk.h>

#include "appconfig.h"
#include "dbapi.h"
#include "export.h"
#include "gtk_callbacks.h"
#include "gtk_config.h"
#include "gtk_export.h"
#include "gtk_interface.h"
#include "gtk_queries.h"
#include "gtk_support.h"
#include "gtk_treeview.h"
#include "gtkui-object.h"
#include "gui_gtk.h"
#include "guiapi.h"
#include "scriptapi.h"

#include "Pics/warning.xpm"
#include "Pics/question.xpm"

/** Call data for dialog.	*/
typedef struct _call_data {
	GtkWidget *dialog;						/**< Dialog widget.	*/
	void (*user_ok_callback) (void *);		/**< Callback for OK button.	*/
	void (*user_cancel_callback) (void *);	/**< Callback for Cancel button.	*/
	void *user_data;						/**< user data.	*/
} CallData;

/** The predefined standard dialog types.	*/
typedef enum {
	DLGTYPE_NONE = 0,	/**< Not a predefined dialog.	*/
	DLGTYPE_WARNING,	/**< A warning dialog.	*/
	DLGTYPE_QUESTION	/**< A question dialog.	*/
} DlgType;


typedef struct _activator {
	GtkWidget *widget;		/**< The widget to activate.	*/
	int (*activator) ();	/**< Test function.	If this function returns non-zero, the widget is activated.	*/
	struct _activator *next;
} Activator;

struct _GtkuiObjectPriv {
	Activator *activators;

	GtkWidget *aboutDlg;
	GtkWidget *configDlg;
	GtkWidget *mainWindow;
	GtkWidget *exportDlg;
	GtkWidget *exportDlgHtmlRadio;
	GtkWidget *exportDlgHtmlRecordsPerPage;
	GtkWidget *exportDlgTextRadio;
	GtkWidget *exportDlgTextSeparator;
	GtkWidget *menuEdit;
	GtkWidget *menuFile;
	GtkWidget *menuHelp;
	GtkWidget *menuView;
	GtkWidget *mnuAbout;
	GtkWidget *mnuClear;
	GtkWidget *mnuClose;
	GtkWidget *mnuCopy;
	GtkWidget *mnuConnect;
	GtkWidget *mnuCut;
	GtkWidget *mnuDisconnect;
	GtkWidget *mnuExport;
	GtkWidget *mnuNew;
	GtkWidget *mnuOpen;
	GtkWidget *mnuPaste;
	GtkWidget *mnuPrintResults;
	GtkWidget *mnuSaveAs;
	GtkWidget *mnuSend;

	GtkWidget *mnuViewRefresh;
	GtkWidget *mnuViewToolbar;
	GtkWidget *mnuViewTreeview;

	GtkWidget *openFileDialog;

	GtkWidget *queryNotebook;

	GtkWidget *queryPopupClear;
	GtkWidget *queryPopupClose;
	GtkWidget *queryPopupSave;
	GtkWidget *queryPopupSaveAs;
	GtkWidget *queryPopupSend;

	GtkWidget *querySaveDialog;

	GtkWidget *resultPopup;
	GtkWidget *resultPopupExport;
	GtkWidget *resultPopupPrint;

	GtkWidget *queryImportDlg;
	GtkWidget *querySaveDlg;
	GtkWidget *querySavePromptDlg;
	GtkWidget *scriptOpenDlg;
	GtkWidget *scriptSaveDlg;
	GtkWidget *treeScroller;
	GtkWidget *treeView;

	GtkWidget *tablePopup;

	GtkWidget *toolbarHandlebox;
	GtkWidget *tbConnectBtn;
	GtkWidget *tbCloseBtn;
	GtkWidget *tbDisconnectBtn;
	GtkWidget *tbExportBtn;
	GtkWidget *tbLoadBtn;
	GtkWidget *tbNewBtn;
	GtkWidget *tbRefreshBtn;
	GtkWidget *tbSaveAsBtn;
	GtkWidget *tbSendBtn;
	GtkWidget *toolbarHolder;
	GtkWidget *treePopup;
	GtkWidget *treePopupDelete;
	GtkWidget *treePopupInsert;
	GtkWidget *treePopupRefresh;
	GtkWidget *treePopupSelect;
	GtkWidget *treePopupUpdate;

	/* Widgets that are enabled when we're connected/disconnected. */
	GList *enabled_connected_widgets;
	GList *enabled_disconnected_widgets;
};

typedef struct {
	GCompletion *completion;
	GList *names;

	guint complete_idle_id;
	gboolean touched;

	GtkWidget *combo;
	GtkWidget *entry;
	GtkWidget *dialog;
} CompleteNameData;

enum {
	CONNECTED,
	DISCONNECTED,
	LAST_SIGNAL
};

//static guint signals[LAST_SIGNAL] = { 0 };

static void gtkui_class_init(GtkuiObjectClass * klass);

static GObjectClass *parent_class = NULL;
GtkuiObject *gtkapp = NULL;

void gtkui_add_activator(GtkuiObject * app, GtkWidget *widget, int (*tester)()) {
	Activator *activator;
	Activator *list;

	if (!app)
		return;

	if (!app->priv)
		return;

	activator = g_malloc(sizeof(Activator));
	if (activator) {
		activator->widget = widget;
		activator->activator = tester;
		activator->next = NULL;
	}

	list = app->priv->activators;
	if (list) {
		/* add to existing list	*/
		while (list->next)
			list = list->next;

		list->next = activator;
	}
	else {
		/* no list.	*/
		app->priv->activators = activator;
	}
}

int result_export_ok(void) {
	return 1;
}

GtkWidget *gtkui_get_queryNotebook(GtkuiObject * app)
{
	if (!app)
		return NULL;

	if (!app->priv)
		return NULL;

	return app->priv->queryNotebook;
}

/** Load the export options dialog from the config data.	*/
void gtkui_exportoptions_load_config(GtkuiObject * app)
{
	if (!app)
		return;

	if (!app->priv)
		return;

	if (app->priv->exportDlgTextSeparator)
		gtk_entry_set_text(GTK_ENTRY(app->priv->exportDlgTextSeparator),
						   exportoptions_get_separator());
	else
		g_warning("exportoptions_load_dlg(): txtExportSeparator NULL");

	if (app->priv->exportDlgHtmlRecordsPerPage)
		gtk_spin_button_set_value(GTK_SPIN_BUTTON
								  (app->priv->exportDlgHtmlRecordsPerPage),
								  exportOptions.htmlRecordsPerPage);
	else
		g_warning
			("exportoptions_load_dlg(): htmlExportRecordsPerPageselect * from NULL");
}

GtkWidget *gtkui_get_configDlg(GtkuiObject * app)
{
	if (!app)
		return NULL;

	if (!app->priv)
		return NULL;

	if (!app->priv->configDlg)
		app->priv->configDlg = create_configDlg();

	return app->priv->configDlg;
}

GtkWidget *gtkui_get_exportDlgHtmlRadio(GtkuiObject * app)
{
	if (!app)
		return NULL;

	if (!app->priv)
		return NULL;

	return app->priv->exportDlgHtmlRadio;
}

GtkWidget *gtkui_get_exportDlgHtmlRecordsPerPage(GtkuiObject * app)
{
	if (!app)
		return NULL;

	if (!app->priv)
		return NULL;

	return app->priv->exportDlgHtmlRecordsPerPage;
}

GtkWidget *gtkui_get_exportDlgTextRadio(GtkuiObject * app)
{
	if (!app)
		return NULL;

	if (!app->priv)
		return NULL;

	return app->priv->exportDlgTextRadio;
}

GtkWidget *gtkui_get_exportDlgTextSeparator(GtkuiObject * app)
{
	if (!app)
		return NULL;

	if (!app->priv)
		return NULL;

	return app->priv->exportDlgTextSeparator;
}

GtkWidget *gtkui_get_mnuViewTreeview(GtkuiObject * app)
{
	if (!app)
		return NULL;

	if (!app->priv)
		return NULL;

	return app->priv->mnuViewTreeview;
}

GtkWidget *gtkui_get_mnuViewToolbar(GtkuiObject * app)
{
	if (!app)
		return NULL;

	if (!app->priv)
		return NULL;

	return app->priv->mnuViewToolbar;
}

GtkWidget *gtkui_get_openFileDialog(GtkuiObject * app)
{
	if (!app)
		return NULL;

	if (!app->priv)
		return NULL;

	if (!app->priv->openFileDialog)
		app->priv->openFileDialog = create_fileOpenDlg();

	return app->priv->openFileDialog;
}

GtkWidget *gtkui_get_querySaveDlg(GtkuiObject * app)
{
	if (!app)
		return NULL;

	if (!app->priv)
		return NULL;

	if (!app->priv->querySaveDlg)
		app->priv->querySaveDlg = create_querySaveDlg();

	return app->priv->querySaveDlg;
}

GtkWidget *gtkui_get_resultPopup(GtkuiObject * app)
{
	if (!app)
		return NULL;

	if (!app->priv)
		return NULL;

	if (!app->priv->resultPopup)
		app->priv->resultPopup = create_resultPopup();

	return app->priv->resultPopup;
}

GtkWidget *gtkui_get_toolbar(GtkuiObject * app)
{
	if (!app)
		return NULL;

	if (!app->priv)
		return NULL;

	return app->priv->toolbarHolder;
}

GtkWidget *gtkui_get_toolbar_menuitem(GtkuiObject * app)
{
	if (!app)
		return NULL;

	if (!app->priv)
		return NULL;

	return app->priv->mnuViewToolbar;
}

GtkWidget *gtkui_get_treeview(GtkuiObject * app)
{
	if (!app)
		return NULL;

	if (!app->priv)
		return NULL;

	return app->priv->treeView;
}

GtkWidget *gtkui_get_treeview_menuitem(GtkuiObject * app)
{
	if (!app)
		return NULL;

	if (!app->priv)
		return NULL;

	return app->priv->mnuViewTreeview;
}

GtkWidget *gtkui_get_treePopup(GtkuiObject * app)
{
	if (!app)
		return NULL;

	if (!app->priv)
		return NULL;

	if (!app->priv->treePopup)
		app->priv->treePopup = create_treePopup();

	return app->priv->treePopup;
}

GtkWidget *gtkui_get_treeView(GtkuiObject * app)
{
	if (!app)
		return NULL;

	if (!app->priv)
		return NULL;

	return app->priv->treeView;
}

static void gtkui_init(GtkuiObject * singleton_app);

GType gtkui_object_get_type(void)
{
	static GType object_type = 0;

	if (!object_type) {
		static const GTypeInfo object_info = {
			sizeof(GtkuiObjectClass),
			NULL,					   /* base_init */
			NULL,					   /* base_finalize */
			(GClassInitFunc) gtkui_class_init,
			NULL,					   /* class_finalize */
			NULL,					   /* class_data */
			sizeof(GtkuiObject),
			0,						   /* n_preallocs */
			(GInstanceInitFunc) gtkui_init,
		};

		object_type =
			g_type_register_static(G_TYPE_OBJECT, "GtkuiObject", &object_info,
								   0);
	}
	return object_type;
}

void gtkui_set_query_name(GtkuiObject * app, GtksqlQueryScintilla * qry,
						  const gchar * name)
{
	GtkWidget *page;
	int position;

	if (!app)
		return;

	if (!app->priv)
		return;

	if (!qry)
		return;

	if ((name == NULL) || (strlen(name) == 0))
		return;

	/* Set the notebook labels if the query is in the notebook.  */
	position =
		gtk_notebook_page_num(GTK_NOTEBOOK(app->priv->queryNotebook),
							  GTK_WIDGET(qry));
	if (position > -1) {
		page =
			gtk_notebook_get_nth_page(GTK_NOTEBOOK(app->priv->queryNotebook),
									  position);
		gtk_notebook_set_tab_label_text(GTK_NOTEBOOK
										(app->priv->queryNotebook), page,
										name);
		gtk_notebook_set_menu_label_text(GTK_NOTEBOOK
										 (app->priv->queryNotebook), page,
										 name);
	}
}

void gtkui_treeview_show(GtkuiObject * app)
{
	if (!app)
		return;

	if (!app->priv)
		return;

	if (!app->priv->treeScroller)
		return;

	gtk_widget_show(app->priv->treeScroller);
}

static void gtkui_show_url(GtkMenuItem *item, gpointer data)
{
	ABuf *buf = buf_new(1024);
	const gchar *url;

	if (! data)
		return;

	buf = buf_strcpy(buf, "mozilla ");

	url = dbconnector_get_help_url((DBConnector *)data);
	if (url)
		buf = buf_strcat(buf, url);

	buf = buf_strcat(buf, "&");

	/** \todo Run a generic browser command.	*/
	/** \todo Add a config file item for browser command.	*/
	system((char *)buf->b_dat);
	buf = buf_free(buf);
}

static void app_finalize(GObject * object);

void gtkui_append_filemenu_item(GtkuiObject * app, GtkWidget *item) {
	GtkWidget *menu;

	if (! app)
		return;

	if (! app->priv)
		return;

	if (! app->priv->menuFile)
		return;

	/* menuFile is a MenuItem. */
	menu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(app->priv->menuFile));
	if (! menu)
		return;

	if (! item)
		return;

	gtk_widget_show (item);
	gtk_container_add(GTK_CONTAINER (menu), item);
}

static void gtkui_class_init(GtkuiObjectClass * klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS(klass);
	parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));
	object_class->finalize = app_finalize;
	/* 
	 * signals[CONNECTED] = g_signal_new("connected",
	 * G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST, 0, NULL, NULL,
	 * gossip_marshal_VOID__VOID, G_TYPE_NONE, 0); signals[DISCONNECTED] =
	 * g_signal_new("disconnected", G_TYPE_FROM_CLASS(klass),
	 * G_SIGNAL_RUN_LAST, 0, NULL, NULL, gossip_marshal_VOID__VOID,
	 * G_TYPE_NONE, 0); */
}

/** 'OK' callback for connection dialog.	*/
static void connection_dialog_ok_callback(GtkWidget * widget, gpointer data)
{
	CallData *data_rec = (CallData *) data;

	if (data_rec->user_ok_callback != NULL)
		data_rec->user_ok_callback(data_rec->user_data);

	gtk_widget_destroy(GTK_WIDGET(data_rec->dialog));
	g_free(data);
}

/** 'Cancel' callback for connection dialog.	*/
static void connection_dialog_cancel_callback(GtkWidget * widget, gpointer data)
{
	CallData *data_rec = (CallData *) data;

	if (data_rec->user_cancel_callback != NULL)
		data_rec->user_cancel_callback(data_rec->user_data);

	gtk_widget_destroy(GTK_WIDGET(data_rec->dialog));
	g_free(data);
}

/** Creates a default dialog.
 * \param title	The title text (Optional).
 * \param text	The dialog text (Optional).
 * \param predefined_type	One of the DlgType values.
 * \param pixmap	Pixmap to display for DLGTYPE_NONE (NULL if unused).
 * \param ok_text	The text to display on the "OK" button. Optional. If NULL,
 * no OK button will be displayed.
 * \param ok_callback	The callback for the OK button.
 * \param cancel_text	The text for the Cancel button. Optional. If NULL,
 * no Cancel button will be displayed.
 * \param cancel_callback	The callback function for the Cancel button.
 * \param callback_data	The user data to pass to the callback functions.	*/
static GtkWidget *gtkui_create_standard_dialog(const char *title, const char *text,
								  const DlgType predefined_type,
								  char **pixmap, const char *ok_text,
								  void (*ok_callback) (void *),
								  const char *cancel_text,
								  void (*cancel_callback) (void *),
								  void *callback_data)
{
	GtkWidget *dialog, *table, *label, *ok_button, *cancel_button;
	CallData *data_send;
	GtkWidget *lbl;

	switch (predefined_type) {
		case DLGTYPE_WARNING:
			pixmap = warning_xpm;
			if (title == NULL)
				title = "Warning";
			break;

		case DLGTYPE_QUESTION:
			pixmap = question_xpm;
			if (title == NULL)
				title = "Question";
			break;

		default:
			break;
	}

	if (title == NULL)
		title = "Message";

	if (text == NULL)
		text = "";

	/* The dialog */
	dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(dialog), title);
	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_MOUSE);
	gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
	gtk_grab_add(dialog);
	gtk_widget_realize(dialog);

	/* Now, the top half (pixmap + text) */
	table = gtk_table_new(2, 1, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table), 5);
	gtk_table_set_col_spacings(GTK_TABLE(table), 5);

	if (pixmap != NULL) {
		GdkBitmap *mask;
		GdkPixmap *gdkpix;
		GtkStyle *style;
		GtkWidget *gtkpix;

		style = gtk_widget_get_style(dialog);
		gdkpix = gdk_pixmap_create_from_xpm_d(dialog->window,
											  &mask,
											  &style->bg[GTK_STATE_NORMAL],
											  pixmap);
		gtkpix = gtk_image_new_from_pixmap(gdkpix, mask);
		gtk_table_attach_defaults(GTK_TABLE(table), gtkpix, 0, 1, 0, 1);
		gtk_widget_show(gtkpix);
	}

	label = gtk_label_new(text);
	gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 0, 1);
	gtk_widget_show(label);

	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), table, TRUE,
					   TRUE, 0);
	gtk_widget_show(table);

	/* The structure for the callbacks */
	data_send = (CallData *) g_malloc(sizeof(CallData));
	data_send->dialog = dialog;
	data_send->user_data = callback_data;
	data_send->user_ok_callback = ok_callback;
	data_send->user_cancel_callback = cancel_callback;

	/* The bottom half */
	if (ok_text != NULL) {
		if (strcmp(ok_text, "OK") == 0)
			ok_button = gtk_button_new_from_stock(GTK_STOCK_OK);
		else {
			lbl = gtk_label_new(ok_text);
			gtk_misc_set_padding(GTK_MISC(lbl), 4, 0);
			gtk_widget_show(lbl);
			ok_button = gtk_button_new();
			gtk_container_add(GTK_CONTAINER(ok_button), lbl);
		}
		g_signal_connect(G_OBJECT(ok_button), "clicked", G_CALLBACK(connection_dialog_ok_callback), data_send);

		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
						   ok_button, FALSE, FALSE, 0);
		GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT);
		gtk_widget_grab_default(ok_button);
		gtk_widget_show(ok_button);
	}

	if (cancel_text != NULL) {
		if (strcmp(cancel_text, "Cancel") == 0)
			cancel_button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
		else {
			lbl = gtk_label_new(cancel_text);
			gtk_misc_set_padding(GTK_MISC(lbl), 4, 0);
			gtk_widget_show(lbl);
			cancel_button = gtk_button_new();
			gtk_container_add(GTK_CONTAINER(cancel_button), lbl);
		}
		g_signal_connect(G_OBJECT(cancel_button), "clicked", G_CALLBACK(connection_dialog_cancel_callback), data_send);

		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
						   cancel_button, FALSE, FALSE, 0);
		gtk_widget_show(cancel_button);
	}

	return dialog;
}

/** Creates a confirmation dialog.
 * \param title	The title text (Optional).
 * \param text	The dialog text (Optional).
 * \param ok_text	The text to display on the "OK" button. Optional. If NULL,
 * no OK button will be displayed.
 * \param ok_callback	The callback for the OK button.
 * \param cancel_text	The text for the Cancel button. Optional. If NULL,
 * no Cancel button will be displayed.
 * \param cancel_callback	The callback function for the Cancel button.	*/
GtkWidget *gtkui_create_question_dialog(const char *title, const char *text,
								  const char *ok_text, void (*ok_callback) (void *),
								  const char *cancel_text, void (*cancel_callback) (void *), void *data)
{
	return gtkui_create_standard_dialog(title, text, DLGTYPE_QUESTION, NULL,
									ok_text, ok_callback, cancel_text, NULL, data);
}

/** Creates a confirmation dialog.
 * \param title	The title text (Optional).
 * \param text	The dialog text (Optional).	*/
GtkWidget *gtkui_create_error_dialog(const char *title, const char *text)
{
	return gtkui_create_standard_dialog(title, text, DLGTYPE_WARNING, NULL, "OK", NULL, NULL, NULL, NULL);
}

void gtkui_disconnect(GtkuiObject * app)
{
	DBConnection *conn = ui_get_connection();

	if (conn == NULL) {
		g_warning("gtkgui_disconnect(): No connection to disconnect.\n");
		return;
	}
	ui_delete_all_queries();

	/* Resets the queries counter */
	queries_init();
	dbconnection_disconnect(conn, NULL);
	ui_set_connection(NULL);

	/* Update gui */
	gtk_window_set_title(GTK_WINDOW(app->priv->mainWindow), "GtkSQL");
	gtkui_update(app, FALSE);
}

/** Callback for the 'Disconnect' dialog.
 * Resets all the display items.
 * \param data	Unused.	*/
void gtkui_disconnect_callback(void *data)
{
	if (!gtkapp) {
		g_warning("gtkui_disconnect_callback(%p): gtkapp is NULL\n", data);
		return;
	}

	gtkui_disconnect(gtkapp);
	gtk_window_set_title(GTK_WINDOW(gtkapp->priv->mainWindow), "GtkSQL");
	gtkui_update(gtkapp, FALSE);
}

void gtkui_do_export(GtkuiObject * app, GtksqlQueryScintilla * query, char *filename, gboolean asText)
{
	DoExportData dedata;

	if (!app)
		return;

	if (!app->priv)
		return;

	dedata.query = query;
	dedata.caption = (char *) gtksql_query_scintilla_get_text(query);
	dedata.conn = ui_get_connection();
	dedata.filename = filename;
	dedata.raw_filename = filename;
	dedata.result = gtksql_query_scintilla_get_result(query);

	if (((dedata.filename == NULL) || (strlen(dedata.filename) == 0)) ||
		((dedata.raw_filename == NULL)
		 || (strlen(dedata.raw_filename) == 0))) {
		ui_show_error(_("Error during result export"),
					  _("No filename specified."));
		return;
	}

	if (asText) {
		/* Text export */
		do_export_text(&dedata);
	}
	else {
		/* HTML export */
		do_export_html(&dedata);
	}
}

void gtkui_do_load_from_file(GtkuiObject * app)
{
	const gchar *filename, *raw_filename;
	ABuf *buf = buf_new(256);
	FILE *fp;
	GtksqlQueryScintilla *cq;

	if (!app)
		return;

	if (!app->priv)
		return;

	cq = query_get_current();

	if (cq == NULL)
		return;

	ui_set_query_text("");

	/* We get the file name */
	filename =
		gtk_file_selection_get_filename(GTK_FILE_SELECTION
										(app->priv->queryImportDlg));
	raw_filename =
		gtk_entry_get_text(GTK_ENTRY
						   (GTK_FILE_SELECTION
							(app->priv->queryImportDlg)->selection_entry));

	fp = fopen(filename, "r");
	if (fp != NULL) {
		while (fgets((char *)buf->b_dat, buf->b_len, fp) != NULL)
			gtksql_query_scintilla_append_text(cq, (char *)buf->b_dat);

		fclose(fp);
		ui_set_query_name((char *) filename);
	}
	buf_free(buf);
	gtk_widget_hide(GTK_WIDGET(app->priv->queryImportDlg));
}

void gtkui_do_save_to_file(GtkuiObject * app, GtksqlQueryScintilla * query)
{
	gchar *filename;

	if (!app)
		return;

	if (!app->priv)
		return;

	if (!app->priv->querySaveDlg)
		return;

	filename = (gchar *)
		gtk_file_selection_get_filename(GTK_FILE_SELECTION
										(app->priv->querySaveDlg));

	query_save_to_file(query, (char *) filename);
	gtk_widget_hide(GTK_WIDGET(app->priv->querySaveDlg));
}

GtkWidget *gtkui_get_mainWindow(GtkuiObject * app)
{
	if (!app)
		return NULL;

	if (!app->priv)
		return NULL;

	return app->priv->mainWindow;
}

void gtkui_hide_aboutDlg(GtkuiObject * app)
{
	if (!app)
		return;

	if (!app->priv)
		return;

	if (app->priv->aboutDlg)
		gtk_widget_hide(GTK_WIDGET(app->priv->aboutDlg));
}

void gtkui_hide_query_import_dialog(GtkuiObject * app)
{
	if (!app)
		return;

	if (!app->priv)
		return;

	if (app->priv->queryImportDlg)
		gtk_widget_hide(GTK_WIDGET(app->priv->queryImportDlg));
}

void gtkui_hide_script_open_dialog(GtkuiObject * app)
{
	if (!app)
		return;

	if (!app->priv)
		return;

	if (app->priv->scriptOpenDlg)
		gtk_widget_hide(GTK_WIDGET(app->priv->scriptOpenDlg));
}

void gtkui_hide_script_save_dialog(GtkuiObject * app)
{
	if (!app)
		return;

	if (!app->priv)
		return;

	if (app->priv->scriptSaveDlg)
		gtk_widget_hide(GTK_WIDGET(app->priv->scriptSaveDlg));
}

void gtkui_hide_treeView(GtkuiObject * app)
{
	if (!app)
		return;

	if (!app->priv)
		return;

	if (app->priv->treeView)
		gtk_widget_hide(GTK_WIDGET(app->priv->treeView));
}

/** Check whether any queries exist.
 * \return	1 if a query exists, 0 otherwise.	*/
int gtkui_query_exists(GtkuiObject * app)
{
	DBConnection *conn = ui_get_connection();
	if (conn == NULL)
		return 0;

	if (query_count() > 0)
		return 1;

	return 0;
}

/** Test if connection is bad.
 * \return	non-zero if the connection is invalid.	*/
static int ui_connection_bad(void)
{
	DBConnection *conn = ui_get_connection();

	if (conn == NULL)
		return 1;
	else
		return 0;
}

/** Check if ready to send a query.
 * \return	1 if a query ready, 0 otherwise.	*/
static int query_ready(void)
{
	if (!gtkui_query_exists(gtkapp))
		return 0;

	return 1;
}

/** Tester to always disable a widget.
 * \returns Always returns 0.	*/
static int always_disabled(void)
{
	return 0;
}

/** Tester to always enable a widget.
 * \returns Always returns 1.	*/
/*static int always_enabled(void)
{
	return 1;
}
*/

/** Add the help menu entries for any drivers that support it.	*/
static void gtkui_helpmenu_init(GtkuiObject * app) {
	DBConnector *db;
	int i;
	const char *url;
	GtkWidget *item;
	GtkMenuShell *menu;
	int doneSeparator = 0;

	if (! app)
		return;

	if (!app->priv)
		return;

	i = 0;
	while ((db = dbconnectors_get(i++))) {
		url = dbconnector_get_help_url(db);
		if (url) {
			menu = GTK_MENU_SHELL(gtk_menu_item_get_submenu(GTK_MENU_ITEM(app->priv->menuHelp)));
			if (! doneSeparator) {
				item = gtk_separator_menu_item_new();
				gtk_menu_shell_append(menu, item);
				gtk_widget_show(item);
				doneSeparator = 1;
			}
			ABuf *buf = buf_new(100);
			buf = buf_strcpy(buf, dbconnector_get_name(db));
			const char *s = dbconnector_get_driver_info(db);
			if (s) {
				buf = buf_strcat(buf, " (");
				buf = buf_strcat(buf, s);
				buf = buf_strcat(buf, ")");				
			}
			item = gtk_menu_item_new_with_label((const gchar *)buf->b_dat);
			buf = buf_free(buf);

			/** \todo Add help signal.	*/
			g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(gtkui_show_url), db);
			gtk_menu_shell_append(menu, item);
			gtk_widget_show(item);
		}
	}
}

void gtkui_load_widgets(GtkuiObject * app)
{
	if (!IS_GTKUI_OBJECT(app))
		return;

	app->priv->toolbarHandlebox = lookup_widget(app->priv->mainWindow, "toolbarHandlebox");
	app->priv->menuEdit = lookup_widget(app->priv->mainWindow, "menuEdit");
	app->priv->menuFile = lookup_widget(app->priv->mainWindow, "menuFile");
	app->priv->menuHelp = lookup_widget(app->priv->mainWindow, "menuHelp");
	app->priv->menuView = lookup_widget(app->priv->mainWindow, "menuView");
	app->priv->mnuAbout = lookup_widget(app->priv->mainWindow, "mnuAbout");
	app->priv->mnuClear = lookup_widget(app->priv->mainWindow, "mnuClear");
	gtkui_add_activator(app, app->priv->mnuClear, gtkui_query_exists);
	app->priv->mnuClose = lookup_widget(app->priv->mainWindow, "mnuClose");
	gtkui_add_activator(app, app->priv->mnuClose, gtkui_query_exists);
	app->priv->mnuConnect = lookup_widget(app->priv->mainWindow, "mnuConnect");
	gtkui_add_activator(app, app->priv->mnuConnect, ui_connection_bad);
	app->priv->mnuCopy = lookup_widget(app->priv->mainWindow, "mnuCopy");
	gtkui_add_activator(app, app->priv->mnuCopy, gtkui_query_exists);
	app->priv->mnuCut = lookup_widget(app->priv->mainWindow, "mnuCut");
	gtkui_add_activator(app, app->priv->mnuCut, gtkui_query_exists);
	app->priv->mnuDisconnect = lookup_widget(app->priv->mainWindow, "mnuDisconnect");
	gtkui_add_activator(app, app->priv->mnuDisconnect, ui_connection_ok);
	app->priv->mnuNew = lookup_widget(app->priv->mainWindow, "mnuNew");
	gtkui_add_activator(app, app->priv->mnuNew, ui_connection_ok);
	app->priv->mnuOpen = lookup_widget(app->priv->mainWindow, "mnuOpen");
	gtkui_add_activator(app, app->priv->mnuOpen, ui_connection_ok);
	app->priv->mnuPaste = lookup_widget(app->priv->mainWindow, "mnuPaste");
	gtkui_add_activator(app, app->priv->mnuPaste, gtkui_query_exists);
	app->priv->mnuSaveAs = lookup_widget(app->priv->mainWindow, "mnuSaveAs");
	gtkui_add_activator(app, app->priv->mnuSaveAs, gtkui_query_exists);
	app->priv->mnuSend = lookup_widget(app->priv->mainWindow, "mnuSend");
	gtkui_add_activator(app, app->priv->mnuSend, query_ready);
	app->priv->mnuViewRefresh = lookup_widget(app->priv->mainWindow, "mnuViewRefresh");
	gtkui_add_activator(app, app->priv->mnuViewRefresh, ui_connection_ok);
	app->priv->mnuViewToolbar = lookup_widget(app->priv->mainWindow, "mnuViewToolbar");
	app->priv->mnuViewTreeview = lookup_widget(app->priv->mainWindow, "mnuViewTreeview");

	app->priv->queryNotebook = lookup_widget(app->priv->mainWindow, "queryNotebook");
	app->priv->querySaveDlg = create_querySaveDlg();
	app->priv->queryImportDlg = create_queryImportDlg();

	app->priv->resultPopup = create_resultPopup();
	if (app->priv->resultPopup != NULL) {
		/* Lookup widgets.  */
		app->priv->resultPopupPrint = lookup_widget(app->priv->resultPopup, "resultPopupPrint");
		gtkui_add_activator(app, app->priv->resultPopupPrint, always_disabled);
	}

	app->priv->tbConnectBtn = lookup_widget(app->priv->mainWindow, "tbConnectBtn");
	gtkui_add_activator(app, app->priv->tbConnectBtn, ui_connection_bad);

	app->priv->tbCloseBtn = lookup_widget(app->priv->mainWindow, "tbCloseBtn");
	gtkui_add_activator(app, app->priv->tbCloseBtn, gtkui_query_exists);

	app->priv->tbDisconnectBtn = lookup_widget(app->priv->mainWindow, "tbDisconnectBtn");
	gtkui_add_activator(app, app->priv->tbDisconnectBtn, ui_connection_ok);

	app->priv->tbExportBtn = lookup_widget(app->priv->mainWindow, "tbExportBtn");
	gtkui_add_activator(app, app->priv->tbExportBtn, gtkui_query_exists);

	app->priv->tbLoadBtn = lookup_widget(app->priv->mainWindow, "tbLoadBtn");
	gtkui_add_activator(app, app->priv->tbLoadBtn, ui_connection_ok);

	app->priv->tbNewBtn = lookup_widget(app->priv->mainWindow, "tbNewBtn");
	gtkui_add_activator(app, app->priv->tbNewBtn, ui_connection_ok);

	app->priv->tbRefreshBtn = lookup_widget(app->priv->mainWindow, "tbRefreshBtn");
	gtkui_add_activator(app, app->priv->tbRefreshBtn, ui_connection_ok);

	app->priv->tbSaveAsBtn = lookup_widget(app->priv->mainWindow, "tbSaveAsBtn");
	gtkui_add_activator(app, app->priv->tbSaveAsBtn, gtkui_query_exists);

	app->priv->tbSendBtn = lookup_widget(app->priv->mainWindow, "tbSendBtn");
	gtkui_add_activator(app, app->priv->tbSendBtn, query_ready);

	app->priv->toolbarHolder = lookup_widget(app->priv->mainWindow, "toolbarHolder");

	app->priv->treeScroller = lookup_widget(app->priv->mainWindow, "treeScroller");
	app->priv->treeView = lookup_widget(app->priv->mainWindow, "treeView");
	app->priv->treePopup = create_treePopup();
	if (app->priv->treePopup != NULL) {
		app->priv->treePopupDelete = lookup_widget(app->priv->treePopup, "treePopupDelete");
		gtkui_add_activator(app, app->priv->treePopupDelete, query_ready);

		app->priv->treePopupInsert = lookup_widget(app->priv->treePopup, "treePopupInsert");
		gtkui_add_activator(app, app->priv->treePopupInsert, query_ready);

		app->priv->treePopupRefresh = lookup_widget(app->priv->treePopup, "treePopupRefresh");
		gtkui_add_activator(app, app->priv->treePopupRefresh, ui_connection_ok);

		app->priv->treePopupSelect = lookup_widget(app->priv->treePopup, "treePopupSelect");
		gtkui_add_activator(app, app->priv->treePopupSelect, query_ready);

		app->priv->treePopupUpdate = lookup_widget(app->priv->treePopup, "treePopupUpdate");
		gtkui_add_activator(app, app->priv->treePopupUpdate, query_ready);
	}
}

static void gtkui_init(GtkuiObject * singleton_app)
{
	GtkuiObjectPriv *priv;

	gtkapp = singleton_app;

	priv = g_new0(GtkuiObjectPriv, 1);
	gtkapp->priv = priv;

	gtkcfg_init();

	/* Window / GTK initialisation */
	gtk_rc_parse(get_rcname());
	add_pixmap_directory(PACKAGE_PIXMAPS_DIR);
	gtkapp->priv->activators = NULL;
	gtkapp->priv->mainWindow = create_mainWindow();
	gtk_widget_show(gtkapp->priv->mainWindow);
	gtkui_load_widgets(gtkapp);
	queries_init();
	treeview_init();
	gtkui_helpmenu_init(gtkapp);

	/* Activate/deactivate the appropriate widgets. */
	gtkui_update(gtkapp, FALSE);
}

/** Set the currently active connection.
 * \param	con	The connection pointer to set.	*/
void gtkui_set_connection(GtkuiObject * app, DBConnection * connection)
{
	char buf[500];
	const char *s = NULL;

	if (!app)
		return;

	treeview_new_connection();
	ui_add_query();
	update_ui();
	if (connection != NULL)
		s = dbconnection_get_db_name(connection);

	if (!s)
		s = "";

	snprintf(buf, sizeof(buf) - 1, "GtkSQL: %s", s);
	buf[sizeof(buf) - 1] = '\0';
	gtk_window_set_title(GTK_WINDOW(app->priv->mainWindow), buf);
}

void gtkui_set_toolbar_visible(GtkuiObject * app, gboolean vis)
{
	if (!app)
		return;

	if (!app->priv)
		return;

	if (app->priv->toolbarHolder) {
		if (vis)
			gtk_widget_show(GTK_WIDGET(app->priv->toolbarHolder));
		else
			gtk_widget_hide(GTK_WIDGET(app->priv->toolbarHolder));
	}
}

void gtkui_show_about_dialog(GtkuiObject * app)
{
/** About dialog text.	*/
#define VERSION_S	_(APPNAME " Version " PACKAGE_VERSION " \nConfigured " CONFDATE)
	ABuf *buf = NULL;
	GtkLabel *lbl;

	if (!app)
		return;

	if (!app->priv)
		return;

	if (app->priv->aboutDlg == NULL) {
		app->priv->aboutDlg = create_aboutDialog();
		buf = buf_strcpy(NULL, "");

#ifdef USE_POSTGRESQL
		buf = buf_strcat(buf, _(" PostgreSQL driver installed \n"));
#endif
#ifdef USE_MYSQL
		buf = buf_strcat(buf, _(" MySQL driver installed \n"));
#endif
#ifdef USE_XPATH
		buf = buf_strcat(buf, _(" XPath driver installed \n"));
#endif
#ifdef USE_SQLITE
		buf = buf_strcat(buf, _(" SQLite driver installed \n"));
#endif

		lbl = GTK_LABEL(lookup_widget
						(GTK_WIDGET
						 (app->priv->aboutDlg), "aboutVersionLabel"));
		if (lbl != NULL)
			gtk_label_set_text(GTK_LABEL(lbl), VERSION_S);

		lbl =
			GTK_LABEL(lookup_widget
					  (GTK_WIDGET(app->priv->aboutDlg), "aboutDriversLabel"));
		if (lbl != NULL)
			gtk_label_set_text(GTK_LABEL(lbl), (char *)buf->b_dat);

		buf_free(buf);
	}
	gtk_widget_show(app->priv->aboutDlg);
}

void gtkui_show_queryImportDlg(GtkuiObject * app)
{
	if (!app)
		return;

	if (!app->priv)
		return;

	if (app->priv->queryImportDlg == NULL)
		app->priv->queryImportDlg = create_queryImportDlg();

	gtk_widget_show(GTK_WIDGET(app->priv->queryImportDlg));
}

static void app_finalize(GObject * object)
{
	GtkuiObject *app;
	GtkuiObjectPriv *priv;

	app = GTKUI_OBJECT(object);
	priv = app->priv;

	g_list_free(priv->enabled_connected_widgets);
	g_list_free(priv->enabled_disconnected_widgets);

	g_free(priv);
	app->priv = NULL;

	if (G_OBJECT_CLASS(parent_class)->finalize)
		(*G_OBJECT_CLASS(parent_class)->finalize) (object);
}

void gtkui_object_connect(void)
{
	GtkuiObjectPriv *priv;

	priv = gtkapp->priv;
}

void gtkui_object_create(void)
{
	/* Create singleton "app". */
	g_object_new(GTKUI_OBJECT_TYPE, NULL);
}

GtkuiObject *gtkui_object_get(void)
{
	g_assert(gtkapp != NULL);

	return gtkapp;
}

void gtkui_treeview_hide(GtkuiObject * app) {
	if (!app)
		return;

	if (!app->priv)
		return;

	gtk_widget_hide(GTK_WIDGET(app->priv->treeScroller));
}

/** Update the displayed widgets to match the current state of the application.	*/
int gtkui_update(GtkuiObject * app, int reportonly)
{
	Activator *activator;
	unsigned int errors = 0;

	if (!IS_GTKUI_OBJECT(app))
		return -1;

	if (! app)
		return -1;

	if (! app->priv)
		return -1;

	activator = app->priv->activators;
	while (activator) {
		if (activator->widget) {
			int act = (activator->activator) ();
			if (reportonly) {
				if (GTK_WIDGET_IS_SENSITIVE(activator->widget) != act)
					errors++;
			}
			else 
				gtk_widget_set_sensitive(GTK_WIDGET(activator->widget), act);
		}
		activator = activator->next;
	}

	if (gtkcfg_get_view_table_tree())
		gtkui_treeview_show(app);
	else
		gtkui_treeview_hide(app);

	return errors;
}

/** OK callback for the file open dialog. The data pointer is the GtkEntry
 * field to receive the filename.	*/
static void file_open_dialog_cancel_callback(GtkWidget * widget,
											 gpointer data)
{
	GtkWidget *openFileDialog = gtkui_get_openFileDialog(gtkapp);

	if (openFileDialog) {
		gtk_grab_remove(openFileDialog);
		gtk_widget_hide(openFileDialog);
	}
}

/** OK callback for the file open dialog. The data pointer is the GtkEntry
 * field to receive the filename.	*/
static void file_open_dialog_ok_callback(GtkWidget * widget, gpointer data)
{
	G_CONST_RETURN gchar *filename;
	GtkWidget *openFileDialog = gtkui_get_openFileDialog(gtkapp);

	if (!openFileDialog)
		return;

	filename =
		gtk_file_selection_get_filename(GTK_FILE_SELECTION(openFileDialog));

	cfg_set_current_dir(filename);
	if (data) {
		gtk_entry_set_text(GTK_ENTRY(data), filename);
	}
	gtk_grab_remove(openFileDialog);
	gtk_widget_hide(openFileDialog);
}

/** Display the export dialog.	*/
void gtkui_show_file_open_dialog(GtkWidget * txt)
{
	GtkWidget *openFileDialog = gtkui_get_openFileDialog(gtkapp);

	if (!openFileDialog)
		if (!(openFileDialog = create_fileOpenDlg()))
			return;

	/* Ensure that the dialog box is destroyed when the user clicks a button. 
	 */
	g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(openFileDialog)->ok_button),
					 "clicked", G_CALLBACK(file_open_dialog_ok_callback),
					 txt);
	g_signal_connect(G_OBJECT
					 (GTK_FILE_SELECTION(openFileDialog)->cancel_button),
					 "clicked", G_CALLBACK(file_open_dialog_cancel_callback),
					 txt);

	/* Display that dialog */
	gtk_grab_add(openFileDialog);
	gtk_widget_show(openFileDialog);
}
