/** \file gtkui-object.h	Experiment with encapsulating global variables in a gobject.
 * \author Darryl Luff
 * \addtogroup uigtk
 * 
 */
/* $Id: gtkui-object.h,v 1.2 2006/02/09 13:37:36 djluff Exp $
 * 
 * GtkSQL -- an interactive graphical query tool for PostgreSQL and MySQL.
 * Copyright (C) 2002-2006 Darryl Luff
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef __GTKUI_OBJECT_H__
#define __GTKUI_OBJECT_H__

#include <glib-object.h>
#include "gtksql_query_scintilla.h"


#define GTKUI_OBJECT_TYPE         (gtkui_object_get_type ())
#define GTKUI_OBJECT(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), GTKUI_OBJECT_TYPE, GtkuiObject))
#define GTKUI_OBJECT_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), GTKUI_OBJECT_TYPE, GtkuiObjectClass))
#define IS_GTKUI_OBJECT(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), GTKUI_OBJECT_TYPE))
#define IS_GTKUI_OBJECT_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), GTKUI_OBJECT_TYPE))
#define GTKUI_OBJECT_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GTKUI_OBJECT_TYPE, GtkuiObjectClass))

typedef struct _GtkuiObject GtkuiObject;
typedef struct _GtkuiObjectClass GtkuiObjectClass;
typedef struct _GtkuiObjectPriv GtkuiObjectPriv;

struct _GtkuiObject {
	GObject parent;

	GtkuiObjectPriv *priv;
};

struct _GtkuiObjectClass {
	GObjectClass parent_class;
};

extern GtkuiObject *gtkapp;
void gtkui_append_filemenu_item(GtkuiObject * app, GtkWidget *item);
GtkWidget *gtkui_create_error_dialog(const char *title, const char *text);
GtkWidget *gtkui_create_question_dialog(const char *title, const char *text,
								  const char *ok_text, void (*ok_callback) (void *),
								  const char *cancel_text, void (*cancel_callback) (void *), void *data);
void gtkui_disconnect(GtkuiObject *app);
void gtkui_disconnect_callback(void *data);
void gtkui_do_export(GtkuiObject * app, GtksqlQueryScintilla * query, char *filename, gboolean asText);
void gtkui_do_load_from_file(GtkuiObject *app);
void gtkui_do_save_to_file(GtkuiObject *app, GtksqlQueryScintilla *query);
void gtkui_exportoptions_load_config(GtkuiObject *app);
GtkWidget *gtkui_get_configDlg(GtkuiObject *app);
GtkWidget *gtkui_get_exportDlgHtmlRadio(GtkuiObject *app);
GtkWidget *gtkui_get_exportDlgHtmlRecordsPerPage(GtkuiObject *app);
GtkWidget *gtkui_get_exportDlgTextRadio(GtkuiObject *app);
GtkWidget *gtkui_get_exportDlgTextSeparator(GtkuiObject *app);
GtkWidget *gtkui_get_mainWindow(GtkuiObject *app);
GtkWidget *gtkui_get_mnuViewToolbar(GtkuiObject *app);
GtkWidget *gtkui_get_mnuViewTreeview(GtkuiObject *app);
GtkWidget *gtkui_get_openFileDialog(GtkuiObject *app);
GtkWidget *gtkui_get_queryNotebook(GtkuiObject *app);
GtkWidget *gtkui_get_querySaveDlg(GtkuiObject *app);
GtkWidget *gtkui_get_resultPopup(GtkuiObject *app);
GtkWidget *gtkui_get_scriptOpenDlg(GtkuiObject * app);
GtkWidget *gtkui_get_scriptSaveDlg(GtkuiObject * app);
GtkWidget *gtkui_get_toolbar(GtkuiObject * app);
GtkWidget *gtkui_get_toolbar_menuitem(GtkuiObject * app);
GtkWidget *gtkui_get_treeview(GtkuiObject * app);
GtkWidget *gtkui_get_treeview_menuitem(GtkuiObject * app);
GtkWidget *gtkui_get_treePopup(GtkuiObject *app);
GtkWidget *gtkui_get_treeView(GtkuiObject *app);
void gtkui_hide_aboutDlg(GtkuiObject *app);
void gtkui_hide_query_import_dialog(GtkuiObject *app);
void gtkui_hide_script_open_dialog(GtkuiObject *app);
void gtkui_hide_script_save_dialog(GtkuiObject *app);
void gtkui_hide_treeView(GtkuiObject *app);
GType gtkui_object_get_type(void) G_GNUC_CONST;
void gtkui_object_create(void);
GtkuiObject *gtkui_object_get(void);
int gtkui_query_exists(GtkuiObject *app);
void gtkui_set_connection(GtkuiObject *app, DBConnection * connection);
void gtkui_set_query_name(GtkuiObject *app, GtksqlQueryScintilla * qry, const gchar * name);
void gtkui_set_toolbar_visible(GtkuiObject *app, gboolean vis);
void gtkui_show_about_dialog(GtkuiObject *app);
void gtkui_show_file_open_dialog(GtkWidget * txt);
void gtkui_show_queryImportDlg(GtkuiObject *app);
void gtkui_show_script_open_dialog(GtkuiObject *app);
void gtkui_show_script_save_dialog(GtkuiObject *app);
int gtkui_update(GtkuiObject *app, int reportonly);

#endif
