/** \file querydata.c	Interface to query data module.
 * \author Lionel ULMER
 * \author Darryl Luff
 * \ingroup uigtk	*/
/* GtkSQL -- an interactive graphical query tool for PostgreSQL
 * Copyright (C) 1998-2004  Lionel ULMER, Darryl Luff.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * ==========================================================================
 *
 */

#include <stdlib.h>
#include <string.h>
#include "querydata.h"

/** Free a QueryData structure.
 * \param qd	The query to free.
 * \return	Always returns NULL.	*/
QueryData *querydata_free(QueryData * qd)
{
	if (qd) {
		if (qd->filename)
			free(qd->filename);
	}
	return NULL;
}

/** Get the filename.
* \param qd	The QueryData structure to access.
* \return	The filename, or NULL.	*/
char *querydata_get_filename(QueryData * qd)
{
	if (!qd)
		return NULL;

	return qd->filename;
}

/** Allocate a new QueryData structure.
 * \return	The QueryData pointer.	*/
void querydata_set_filename(QueryData * qd, char *fn)
{
	if (!qd)
		return;

	if (qd->filename)
		free(qd->filename);

	if (fn)
		qd->filename = strdup(fn);
	else
		qd->filename = NULL;
}

/** Allocate a new QueryData structure.
 * \return	A pointer to the bew QueryData structure.	*/
QueryData *querydata_new()
{
	return (QueryData *) calloc(1, sizeof(QueryData));
}
