/** \addtogroup script Script Module
 * The functions in the script module provide the interface to the script engines.
 *
 *@{
 */
/** \file scriptapi.c	Public script engine API.
 * \author Darryl Luff
 */
/*@}*/
/* GtkSQL -- an interactive graphical query tool for PostgreSQL
 * Copyright (C) 1998-2004 Lionel ULMER, Darryl Luff
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "../config.h"

#include <stdio.h>
#include <stdlib.h>

#include "scriptapi.h"

static ScriptEngine *script = NULL;

void script_config_value(const char *key, const char *val)
{
	if (!script)
		return;

	if (!script->Script_config_value) {
		return;
	}

	script->Script_config_value(script, key, val);
}

/** Dump config to stdout. Not implemented - no config.	*/
void script_dump_config(void)
{
	return;
}

/** Initialise the script engine.	*/
static void script_init_engine(void)
{
	if (!script) {
		fprintf(stderr, "WARNING: script_init_engine() script is NULL\n");
		return;
	}

	if (!script->Script_init) {
		fprintf(stderr,
				"WARNING: script_init_engine() cannot initialise the scripting engine because it does not support initialisation\n");
		return;
	}

	script->Script_init(script);
}

/** Initialise the script environment.	*/
void script_init(void)
{
	if (!script)
		fprintf(stderr, "WARNING: Script support disabled\n");
	else
		script_init_engine();
}

/** Execute a file.	*/
void script_run_file(const char *fn)
{
	if (!script) {
		fprintf(stderr, "WARNING: script_run_file(): script is NULL\n");
		return;
	}

	if (!script->Script_run_file) {
		fprintf(stderr,
				"WARNING: script_run_file(): script engine does not support running files\n");
		return;
	}
	script->Script_run_file(script, fn);
}

/** Execute a string.	*/
void script_run_string(const char *s)
{
	if (!script)
		return;

	if (!script->Script_run_string) {
		fprintf(stderr, "WARNING: Script interface not initialised.\n");
		return;
	}

	script->Script_run_string(script, s);
}

/** Save config lines to the specified file.	*/
void script_save_config(void)
{
	if (!script)
		return;

	if (!script->Script_save_config) {
		fprintf(stderr, "WARNING: Script interface not initialised.\n");
		return;
	}

	script->Script_save_config(script);
}

/** Check if scripting is supported.	*/
int script_supported(void)
{
	if (script)
		return 1;
	else
		return 0;
}

/** Allocate a new script engine.	*/
ScriptEngine *scriptengine_new(void)
{
	return calloc(1, sizeof(ScriptEngine));
}
