/** \addtogroup script
 * Public interface for Script modules.
 *
 *@{
 */
/** \file scriptapi.h	Public script engine API.
 * \author Darryl Luff
 *
 */
/* GtkSQL -- an interactive graphical query tool for PostgreSQL and MySQL.
 * Copyright (C) 1998-2005  Lionel ULMER, Darryl Luff.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of tdo_he License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _SCRIPTAPI_H
#define _SCRIPTAPI_H

/** Per-connection database structure. This defines the interface the
 * application talks to the driver with. It also holds the keyword list
 * for the connection.	*/
typedef struct script_engine ScriptEngine;
/** This is the structure containing the public interface description.	*/
struct script_engine {
	char *name;

	/* Information functions. */
	void (*Script_config_value) (ScriptEngine * script, const char *key, const char *val);	/**< Check the config value.	*/
	void (*Script_init) (ScriptEngine * script);/**< Initialise the script engine.	*/
	void (*Script_run_file) (ScriptEngine * script, const char *fn);	/**< Execute a file.	*/
	void (*Script_run_string) (ScriptEngine * script, const char *s);	/**< Execute a single string.	*/
	const char *(*Script_save_config) (ScriptEngine * script);/**< Get the string needed to set a variable.	*/
	const char *(*Script_set_variable_string) (ScriptEngine * script, const char *var, const char *val);	/**< Get the string needed to set a variable.	*/
	void (*Script_shutdown) (ScriptEngine * script);	/**< Shutdown the engine.	*/
	void *pvt;	/**< Private driver data	*/
};

void script_config_value(const char *key, const char *val);
void script_dump_config(void);
void script_init(void);
void script_run_file(const char *fn);
void script_run_string(const char *s);
void script_save_config(void);
int script_supported(void);
ScriptEngine *scriptengine_new(void);

/*@}*/
#endif
