/** \file utils.c	Utility routines.
 * \author Lionel ULMER
 * \author Darryl Luff
 * \addtogroup misc	*/
/* GtkSQL -- an interactive graphical query tool for PostgreSQL
 * Copyright (C) 2002-2005 Darryl Luff
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/** Get the shortened filename (without the path).
 * \param fn	The full filename to parse.
 * \return	The short filename. This is a pointer into the 'fn' string, so 
 * don't free 'fn' till you've gfinished with this result.	*/
const char *get_short_filename(const char *fn)
{
	const char *sfn;
	char *p;
	if (fn == NULL)
		return NULL;

	if (! fn)
		return "";

	sfn = fn;
	p = strchr(sfn, '/');
	while (p != NULL) {
		sfn = p;
		sfn++;
		p = strchr(sfn, '/');
	}

	/* Now, p should be null, meaning that there are no more '/'s, and 'sfn'
	 * should point to the short name. */
	return sfn;
}

void unimp(const char *s)
{
	if (s)
		fprintf(stderr, "%s not implemented\n", s);
}
