/**
 *
 */

#include <stdio.h>
#include "../src/forms.h"

#define ERRORSTR	"[ERROR]"
#define OKSTR "[OK]"

FormField *test_field_create(char *n, char *v) {
	FormField *newfield;
	char *value;
	char *name;

	// Create a new field
	printf(". Create field: ");
	newfield = fld_new(n);
	if (! newfield) {
		printf("Could not allocate %s\n", ERRORSTR);
		return NULL;
	}
	else
		printf("%s\n", OKSTR);

	// Make sure name is OK
	name = fld_get_name(newfield);
	printf(". Check name: %s ", name);
	if (strcmp(name, n)) {
		newfield = fld_free(newfield);
		printf(" %s\n", ERRORSTR);
		return NULL;
	}
	else
		printf(" %s\n", OKSTR);

	// Make sure value is NULL
	value = fld_get_value(newfield);
	printf(". Check value: %s ", value);
	if (value) {
		printf("%s\n", ERRORSTR);
		newfield = fld_free(newfield);
		return NULL;
	}
	else
		printf("%s\n", OKSTR);

	// Set value
	fld_set_value(newfield, v);
	value = fld_get_value(newfield);
	printf(". Set value: %s = %s ", v, value);
	if (strcmp(v, value)) {
		printf("%s\n", ERRORSTR);
		newfield = fld_free(newfield);
		return NULL;
	}
	else
		printf("%s\n", OKSTR);

	return newfield;
}

int test_field_add(FormField *fld, char *n, char *v) {
	char *value;
	char *name;
	int oldcount;
	int newcount;
	FormField *newfield;

	// Check parameters
	if (! fld)
		return 1;

	oldcount = fld_count(fld);
	printf(". Original count: %d\n", oldcount);

	fld_add(fld, n);

	// Check the value
	newfield = fld_set_value_by_name(fld, n, v);
	value = fld_get_value(newfield);
	printf(". Set value: %s = %s ", v, value);
	if (strcmp(v, value)) {
		printf("%s\n", ERRORSTR);
		return 1;
	}
	else
		printf("%s\n", OKSTR);

	// Check new count
	newcount = fld_count(fld);
	printf(". New count: %d = %d", oldcount + 1, newcount);
	if (newcount != (oldcount + 1)) {
		printf(" %s\n", ERRORSTR);
		return 1;
	}
	else
		printf(" %s\n", OKSTR);

	return 0;
}

int test_fields() {
	FormField *fld = NULL;
	FormField *fld2 = NULL;
	int count;
	int i;
	int n;

	printf("Testing FormField...\n");
	fld = test_field_create("First", "FirstValue");
	if (! fld)
		return 1;

	count = fld_count(fld);
	printf(". Initial fields: %d", count);
	if (count != 1) {
		printf(" %s\n", ERRORSTR);
		return 1;
	}
	else
		printf(" %s\n", OKSTR);

	printf("Adding a field...\n");
	if (test_field_add(fld, "Second", "SecondValue") != 0) {
		return 1;
	}

	count = fld_count(fld);
	printf(". Count: %d", count);
	if (count != 2) {
		printf(" %s\n", ERRORSTR);
		return 2;
	}
	else
		printf(" %s\n", OKSTR);

	return 0;
}

int main() {
	int err;

	err = test_fields();
	if (err)
		return err;

	printf("No errors found.\n");
	return 0;
}
